#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

from __future__ import division

"""
PyAnnHyb see http://bioinformatics.org/pyannhyb for more details about this project

Copyright (C) 2008-2011 Olivier Friard

This file is part of PyAnnHyb.

PyAnnHyb is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

PyAnnHyb is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PyAnnHyb; see the file COPYING.TXT.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
"""

__version__ = '0.021'
version_date = '2011-06-07'


#resource file
import pyannhyb_rc


import os
import sys
import platform
from PyQt4.QtCore import *
from PyQt4.QtGui import *

from open_project import *
from save_project import *

from pyannhyb_ui import Ui_frmPyAnnhyb

from preferences import *
from seq_feature import *
from features_list import *
from seq_selector import *
from alignments_list import *
from re_selection import *
from show_results import *
from new_seq import *

from frmAskValue import *
from translation_parameters import *

from bioseqtools import *
from local_alignment import *
from seq_classes import *
from format import *


tabIndex = {'oligoList':0, 'oligo':1, 'seqList':2, 'seq':3, 'MAList':4}

#annhyb_items={"-1":['Oligonucleotides', tabIndex['oligoList']],"-2":['Sequences', tabIndex['seqList']],"-3":['Multiple alignments',tabIndex['MAList']]}

catType = {oligoList:'oligoList', seqList:'seqList', MAList:'MAList'}

catOrder = ['oligoList', 'seqList', 'MAList']

catList = {'oligoList':{'name':'Oligonucleotides', 'type':oligoList, 'index':None, 'item':None},
           'seqList':{'name':'Sequences', 'type':seqList, 'index':None, 'item':None},
           'MAList':{'name':'Multiple alignments', 'type':MAList, 'index':None, 'item':None}
           }


### dictionary to store TV items: oligo, seq and annotation
tvItems = {}

defaultOligoSaltCc = 50e-3
defaultOligoCc = 250e-9

def monospaceFont(size):
    font = QFont()
    if platform.system() == 'Linux':
        font.setFamily("Monospace")
        font.setPointSize(size)
    elif platform.system() == 'Windows':
        font.setFamily("Courier New")
        font.setPointSize(size)
    elif platform.system() == 'Darwin':
        font.setFamily("Courier New")
        font.setPointSize(size + 2)
    return font


class AnnHyb(QMainWindow):
    projectFileName = ''
    projectDescription = ''
    projectChanged = False

    #initialize dictionaries for oligo, sequence, ma list
    itemList = {}

    preferences = {'mecA':15400, 'mecC':7300, 'mecG':11700, 'mecT':8800, \
    'oligoDefaultSaltConc': 0.05,
    'oligoDefaultConc': 250e-9,
    'seqSelectionDefaultSaltConc': 0.05,
    'seqSelectionDefaultConc': 250e-9, \
    'sequenceDefaultSaltConc': 0.05, \
    'CheckAlignmentCutOff': 90, \
    'checksum':'SEGUID'}


    def __init__(self, parent=None):
        QWidget.__init__(self, parent)

        self.ui = Ui_frmPyAnnhyb()
        self.ui.setupUi(self)
        self.setWindowIcon(QIcon(':/pyannhyb.svg'))
        self.ui.actionQuit.setIcon(QIcon(':/delete.png'))
        self.ui.actionOpen_project.setIcon(QIcon(':/open.png'))
        self.ui.actionSave_project.setIcon(QIcon(':/save.png'))
        self.ui.actionSave_project_as.setIcon(QIcon(':/save.png'))

        self.ui.actionNew_oligo.setIcon(QIcon(':/new.png'))
        self.ui.actionRemove_oligo.setIcon(QIcon(':/delete.png'))

        self.ui.actionNew_sequence.setIcon(QIcon(':/new.png'))
        self.ui.actionOpen_sequence.setIcon(QIcon(':/open.png'))
        self.ui.actionRemove_sequence.setIcon(QIcon(':/delete.png'))
        self.ui.actionSave_sequence.setIcon(QIcon(':/save.png'))
        self.ui.actionHighlight_selected_sequence.setIcon(QIcon(':/annotation.png'))

        self.ui.actionSet_preferences.setIcon(QIcon(':/preferences.png'))
        self.ui.actionSave_project.setIcon(QIcon(':/save.png'))
        self.ui.actionAbout.setIcon(QIcon(':/help.png'))
        self.ui.actionRetrieve_sequence_from_NCBI_server.setIcon(QIcon(':/planet.png'))

        #self.ui.move(100, 100)
        self.setWindowTitle('PyAnnHyb %s (%s)' % (__version__, version_date))

        self.tvInit()
        #self.ui.ta_oligoList.clear()  p.246
        #main form

        #tree view
        QObject.connect(self.ui.tv, SIGNAL("itemSelectionChanged()"), self.tv_selection_changed)

        #oligo events
        QObject.connect(self.ui.le_OligoName, SIGNAL("textEdited(QString)"), self.oligoName_changed)
        QObject.connect(self.ui.le_oligo_seq, SIGNAL("textEdited(QString)"), self.oligoSeq_changed)
        QObject.connect(self.ui.le_salt_cc, SIGNAL("textEdited(QString)"), self.oligoSaltCc_changed)
        QObject.connect(self.ui.le_oligo_cc, SIGNAL("textEdited(QString)"), self.oligoCc_changed)
        QObject.connect(self.ui.le_revcomp, SIGNAL("textEdited(QString)"), self.revcomp_changed)

        #seq events
        #te_seq
        QObject.connect(self.ui.te_seq, SIGNAL("selectionChanged()"), self.te_seq_selectionChanged)
        #QObject.connect(self.ui.te_seq.verticalScrollBar(),SIGNAL("valueChanged(int)"),self.te_seq_valueChanged)
        QObject.connect(self.ui.te_seq.verticalScrollBar(), SIGNAL("sliderReleased()"), self.te_seq_sliderReleased)

        QObject.connect(self.ui.le_seqName, SIGNAL("textEdited(QString)"), self.le_seqName_changed)
        QObject.connect(self.ui.le_seqAuthor, SIGNAL("textEdited(QString)"), self.le_seqAuthor_changed)
        QObject.connect(self.ui.le_longName, SIGNAL("textEdited(QString)"), self.le_longName_changed)
        QObject.connect(self.ui.le_seqDefinition, SIGNAL("textEdited(QString)"), self.le_seqDefinition_changed)
        QObject.connect(self.ui.le_seqDate, SIGNAL("textEdited(QString)"), self.le_seqDate_changed)

        ### menu oligo
        QObject.connect(self.ui.actionNew_oligo, SIGNAL("activated()"), self.newOligo)
        QObject.connect(self.ui.actionRemove_oligo, SIGNAL("activated()"), self.removeItem)
        QObject.connect(self.ui.actionAlign_oligo, SIGNAL("activated()"), self.alignOligo)
        QObject.connect(self.ui.actionExport_oligo_2, SIGNAL("activated()"), self.exportOligoFASTAFormat)
        QObject.connect(self.ui.actionExportOligoData, SIGNAL("activated()"), self.exportOligoData)

        ### menu sequence
        QObject.connect(self.ui.actionNew_sequence, SIGNAL("activated()"), self.newSequence)
        QObject.connect(self.ui.actionOpen_sequence, SIGNAL("activated()"), self.openSequence)
        QObject.connect(self.ui.actionSave_sequence, SIGNAL("activated()"), self.saveSequence)
        QObject.connect(self.ui.actionRemove_sequence, SIGNAL("activated()"), self.removeItem)
        QObject.connect(self.ui.actionRetrieve_sequence_from_NCBI_server, SIGNAL("activated()"), self.retrieveSequence)
        QObject.connect(self.ui.actionHighlight_selected_sequence, SIGNAL("activated()"), self.newSeqAnnotation)
        QObject.connect(self.ui.actionAnnotations_list, SIGNAL("activated()"), self.annotationsList)
        QObject.connect(self.ui.actionRestriction_analysis, SIGNAL("activated()"), self.restrictionAnalysis)
        QObject.connect(self.ui.actionTranslation, SIGNAL("activated()"), self.translation)
        QObject.connect(self.ui.actionCropSequence, SIGNAL("activated()"), self.cropSequence)
        QObject.connect(self.ui.actionRevert_complement, SIGNAL("activated()"), self.revertComplement)

        ### menu tools
        QObject.connect(self.ui.actionCropSequence, SIGNAL("activated()"), self.cropSequence)
        QObject.connect(self.ui.actionDeleteSequence, SIGNAL("activated()"), self.deleteSequence)

        #menu project "File"
        QObject.connect(self.ui.actionOpen_project, SIGNAL("activated()"), self.openProject)
        QObject.connect(self.ui.actionSave_project, SIGNAL("activated()"), self.saveProject)
        QObject.connect(self.ui.actionSave_project_as, SIGNAL("activated()"), self.saveProjectAs)
        QObject.connect(self.ui.actionClose_project, SIGNAL("activated()"), self.closeProject)
        QObject.connect(self.ui.actionQuit, SIGNAL("activated()"), self.quit_application)
        QObject.connect(self.ui.actionSet_preferences, SIGNAL("activated()"), self.setPreferences)
        QObject.connect(self.ui.actionAbout, SIGNAL("activated()"), self.about)


        ### Actions for le_oligo_seq
        self.ui.le_oligo_seq.setContextMenuPolicy(Qt.ActionsContextMenu)

        QObject.connect(self.ui.actionGroupSeq, SIGNAL("activated()"), self.groupSeq)
        self.ui.le_oligo_seq.addAction(self.ui.actionGroupSeq)

        QObject.connect(self.ui.actionUnGroupSeq, SIGNAL("activated()"), self.unGroupSeq)
        self.ui.le_oligo_seq.addAction(self.ui.actionUnGroupSeq)

        QObject.connect(self.ui.actionUPPERCase, SIGNAL("activated()"), self.seqUpperCase)
        self.ui.le_oligo_seq.addAction(self.ui.actionUPPERCase)

        QObject.connect(self.ui.actionLowerCase, SIGNAL("activated()"), self.seqLowerCase)
        self.ui.le_oligo_seq.addAction(self.ui.actionLowerCase)



        ### set monospace font on various platform
        self.ui.te_seq.setFont(monospaceFont(12))
        self.ui.le_oligo_seq.setFont(monospaceFont(12))
        self.ui.le_revcomp.setFont(monospaceFont(12))
        self.ui.le_IUPAC_seq.setFont(monospaceFont(12))

        self.readConfigFile()

        ### system
    if platform.system() == 'Linux':
        NL = '\n'
    if platform.system() == 'Darwin':
        NL = '\r'
    if platform.system() == 'Windows':
        NL = '\r\n'

    def tvInit(self):
        """initialize TreeWidget with list elements"""
        self.ui.tv.clear()
        self.ui.tv.setColumnCount(1)
        self.ui.tv.setHeaderLabels(["Items"])
        #self.ui.tv.setItemsExpandable(True)
        self.ui.tv.setColumnWidth(0, 200)

        for cat in catOrder:
            catList[cat]['item'] = QTreeWidgetItem(self.ui.tv, [catList[cat]['name'], str(catList[cat]['index']), str(catList[cat]['type'])])
            self.ui.tv.expandItem(catList[cat]['item'])



    def groupSeq(self):
        """group oligo sequence by 3 nt"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        s = ''
        for ii in range(len(self.itemList[i].seq.replace(' ', ''))):
            s = s + self.itemList[i].seq.replace(' ', '')[ii]
            if (ii + 1) % 3 == 0:
                s = s + ' '

        self.itemList[i].seq = s
        self.ui.le_oligo_seq.setText(s)
        self.oligoParamUpdate(i)
        self.projectChanged = True


    def unGroupSeq(self):
        """ungroup oligo sequence"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        s = self.itemList[i].seq.replace(' ', '')
        self.itemList[i].seq = s
        self.ui.le_oligo_seq.setText(s)
        self.oligoParamUpdate(i)
        self.projectChanged = True


    def seqUpperCase(self):
        """set sequence to upper case"""
        i = int(self.ui.tv.selectedItems()[0].text(1))

        s = str(self.itemList[i].seq).upper()
        self.itemList[i].seq = s
        self.ui.le_oligo_seq.setText(s)

        self.oligoParamUpdate(i)
        self.projectChanged = True


    def seqLowerCase(self):
        """set sequence to upper case"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        s = str(self.itemList[i].seq).lower()
        self.itemList[i].seq = s
        self.ui.le_oligo_seq.setText(s)
        self.oligoParamUpdate(i)
        self.projectChanged = True



    def te_seq_sliderReleased(self):
        """save sequence vertical scrollbar position"""

        index = int(self.ui.tv.selectedItems()[0].text(1))
        self.itemList[index].memTWpos = self.ui.te_seq.verticalScrollBar().value()

    def checkItem(self):
        """Check selected item type"""
        if self.ui.tv.selectedItems():

            print 'item name:', self.ui.tv.selectedItems()[0].text(0)

            type = int(self.ui.tv.selectedItems()[0].text(2)) ### read item type

            if type == seq_annot_item:  ### seq annotation

                #index = self.itemList[ int(self.ui.tv.selectedItems()[0].text(1)) ].parent_id   ### return id from parent sequence
                index = int(self.ui.tv.selectedItems()[0].text(1))

            elif type in [oligo_item, sequence_item, MSA_item]:
                index = int(self.ui.tv.selectedItems()[0].text(1))

            else:
                index = None
            return {'type':type, 'index':index}

        else:
            #QMessageBox.warning(self,'pyAnnHyb','No item selected')
            return {'type':None, 'index':None}


    """
    def checkSeq(self,list):
        
        msg='Select an item'
        if list==[oligo_item]:
            msg='No selected oligonucleotide'
        if list==[sequence_item]:
            msg='No selected sequence'
            
        if self.ui.tv.selectedItems():
            index=int(self.ui.tv.selectedItems()[0].text(1))  #read index from QTreeWidget
            if self.itemList.has_key(index):
                if self.itemList[index].type in list:
                    return index
                else:
                    QMessageBox.warning(self,"pyAnnHyb",msg)
                    return None
            ### check for sequence annotation    
            elif index==-1 and int(self.ui.tv.selectedItems()[0].text(2))==seqAnnotation and seqAnnotation in list:
                return (-1,seqAnnotation,int(self.ui.tv.selectedItems()[0].text(3)))   ###-1 4 sequence index         
            else:
                QMessageBox.warning(self,"pyAnnHyb",msg)
                return None                
        else:
            QMessageBox.warning(self,"pyAnnHyb",msg)
            return None
    """

    def selectSeq(self, windowTitle, item_type):
        """
        allow user to select sequences
        return list of item index
        """
        frmSeqSelector = SeqSelector()
        frmSeqSelector.setWindowTitle(QString(windowTitle))
        itemCount = 0
        for item in self.itemList:
            if self.itemList[item].type == item_type:
                itemCount += 1

        frmSeqSelector.ui.twSeq.setRowCount(itemCount)
        frmSeqSelector.ui.twSeq.setSortingEnabled(False)

        row = 0

        for item in self.itemList:
            if self.itemList[item].type == item_type:
                seq = self.itemList[item]
                #item.setTextAlignment(Qt.AlignCenter)
                #frmSeqSelector.ui.twSeq.setItem(row, 0, QTableWidgetItem(str(seq.name)))
                tw_item = QTableWidgetItem(str(seq.name))

                tw_item.setData(Qt.UserRole, QVariant(item))

                tw_item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable)
                tw_item.setCheckState(Qt.Unchecked) #   Qt.Checked

                #display checkbox    
                frmSeqSelector.ui.twSeq.setItem(row, 0, tw_item)

                """
                frmSeqSelector.ui.twSeq.setItem(row, 1, QTableWidgetItem(str(feature.begin)))
                frmSeqSelector.ui.twSeq.setItem(row, 2, QTableWidgetItem(str(feature.end)))
                frmSeqSelector.ui.twSeq.setItem(row, 3, QTableWidgetItem(str(feature.comment)))
                """

                row += 1

        frmSeqSelector.ui.twSeq.resizeColumnsToContents()

        r = frmSeqSelector.exec_()

        print 'select seq result:', r
        if not r:
            return []

        selectedSeq = []
        for row in range(frmSeqSelector.ui.twSeq.rowCount()):
            if frmSeqSelector.ui.twSeq.item(row, 0):
                #print 'question:',questionDialog.ui.twChoices.item(r,0).text()

                answer = frmSeqSelector.ui.twSeq.item(row, 0)
                if answer.checkState() == Qt.Checked:
                    #print answer.data(Qt.UserRole).toInt()[0]
                    selectedSeq.append(answer.data(Qt.UserRole).toInt()[0])

        #print selectedSeq
        return selectedSeq


    def alignOligo(self):
        """Local alignment of selected oligo against all sequences"""

        #item = self.checkItem()

        ### select oligo 
        selectedOligo = self.selectSeq('Select oligonucleotides to align', oligo_item)
        if not selectedOligo:
            return

        ### select sequences
        selectedSeq = self.selectSeq('Select sequences', sequence_item)
        if not selectedSeq:
            return

        for k in selectedOligo:   ### oligo iter

                seq1 = self.itemList[k].seq.replace(' ', '')
                seq1_revcomp = rev_comp_seq(seq1)

                all_la_results = []

                for j in selectedSeq:

                        #sense
                        #seq2 = self.itemList[j].seq
                        print 'init local alignment'
                        la_results = local_alignment(self.itemList[j].seq, seq1)

                        print 'local alignment', la_results
                        for la_result in la_results:
                            all_la_results += [{'sense':'+',
                                                'query_name': self.itemList[k].name,
                                                'subject_name': self.itemList[j].name,
                                                'subject_id': j,
                                                'query': la_result['aligned_query_seq'],
                                                'match': la_result['match'],
                                                'subject': la_result['aligned_subject_seq'],
                                                'identity': la_result['score'],
                                                'query_start': 0,
                                                'query_end': 0,
                                                'subject_start': la_result['init_alignment'],
                                                'subject_end': la_result['end_alignment']}]

                        #reverse complement
                        la_results = local_alignment(self.itemList[j].seq, seq1_revcomp)
                        print 'local alignment', la_results
                        for la_result in la_results:
                            all_la_results += [{'sense':'-',
                                              'query_name':self.itemList[k].name,
                                                'subject_name':self.itemList[j].name,
                                                'subject_id':j,
                                                'query':la_result['aligned_query_seq'],
                                                'match':la_result['match'],
                                                'subject':la_result['aligned_subject_seq'],
                                                'identity':la_result['score'],
                                                'query_start':0,
                                                'query_end':0,
                                                'subject_start':la_result['init_alignment'],
                                                'subject_end':la_result['end_alignment']}]

                frmAlignmentsList = AlignmentsList()
                frmAlignmentsList.ui.twAlignmentsList.setRowCount(len(all_la_results))
                frmAlignmentsList.ui.twAlignmentsList.setSortingEnabled(False)

                twAlignmentsListPos = {'display':0, 'subject_name':1, 'sense':2, 'identity':3, 'subject_start':4, 'subject_end':5, 'alignment':6}

                for index, alignment in enumerate(all_la_results):

                        #checkbox inside QTabbleWidget
                        item = QTableWidgetItem('')
                        item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable)
                        """
                        print alignment,type(alignment)
                        print 'CheckAlignmentCutOff',self.preferences['CheckAlignmentCutOff'],type(self.preferences['CheckAlignmentCutOff'])
                        print  alignment['identity'],type(alignment['identity'])
                        """
                        if alignment['identity'] >= float(self.preferences['CheckAlignmentCutOff']):
                            item.setCheckState(Qt.Checked)
                        else:
                            item.setCheckState(Qt.Unchecked)
                        item.setData(Qt.UserRole, QVariant(index))
                        #display checkbox    
                        frmAlignmentsList.ui.twAlignmentsList.setItem(index, twAlignmentsListPos['display'], item)

                        for field in ['subject_name', 'sense', 'identity', 'subject_start', 'subject_end']:
                            item = QTableWidgetItem(str(alignment[field]))
                            item.setTextAlignment(Qt.AlignCenter)

                            frmAlignmentsList.ui.twAlignmentsList.setItem(index, twAlignmentsListPos[field], item)

                        #show alignment
                        item = QTableWidgetItem(alignment['query'] + '\n' + alignment['match'] + '\n' + alignment['subject'])
                        item.setFont(monospaceFont(10))

                        frmAlignmentsList.ui.twAlignmentsList.setItem(index, twAlignmentsListPos['alignment'], item)
                        frmAlignmentsList.ui.twAlignmentsList.setRowHeight(index, 80)

                frmAlignmentsList.ui.twAlignmentsList.resizeColumnsToContents()

                frmAlignmentsList.setWindowTitle(QString('Local alignment results for ' + self.itemList[k].name))

                frmAlignmentsList.exec_()

                #add checked alignments to annotations
                for row in range(0, frmAlignmentsList.ui.twAlignmentsList.rowCount()):
                    #read checkbox
                    display = frmAlignmentsList.ui.twAlignmentsList.item(row, 0)
                    index = display.data(Qt.UserRole).toInt()[0]
                    if display.checkState() == Qt.Checked:
                        self.projectChanged = True
                        seq_feat = Seq_Feat()
                        subject = all_la_results[index]['subject_name']
                        #subject_id = all_la_results[index]['subject_id']
                        seq_feat.name = self.itemList[k].name  #oligo name
                        seq_feat.comment = str(seq_feat.name + ' local alignment')
                        seq_feat.begin = all_la_results[index]['subject_start']
                        seq_feat.end = all_la_results[index]['subject_end']
                        seq_feat.parent_id = all_la_results[index]['subject_id']

                        identity = all_la_results[index]['identity']
                        if identity >= 95:
                            seq_feat.color = (255, 0, 0, 255) #insert color gradient based on identity?
                        elif identity >= 90:
                            seq_feat.color = (255, 64, 64, 255) #insert color gradient based on identity?                            
                        elif identity >= 85:
                            seq_feat.color = (255, 128, 128, 255) #insert color gradient based on identity?
                        else:
                            seq_feat.color = (255, 172, 172, 255) #insert color gradient based on identity?


                        item_id = len(self.itemList) + 1
                        self.itemList[item_id] = seq_feat

                        newFeature = QTreeWidgetItem(
                                                 tvItems[seq_feat.parent_id],
                                                 [seq_feat.name, str(item_id), str(seq_annot_item)]
                                                 )

                        self.ui.tv.expandItem(tvItems[seq_feat.parent_id])
                        newFeature.setSelected(True)
                        tvItems[seq_feat.parent_id].setSelected(False)

                        ### set background color for annotation
                        colorFeature = QColor(seq_feat.color[0], seq_feat.color[1], seq_feat.color[2], seq_feat.color[3])
                        newFeature.setBackgroundColor(0, colorFeature)

                        tvItems[item_id] = newFeature

                        #self.itemList[item['index']].memFeatures=[]
                        self.projectChanged = True

                        self.insertAnnotations(seq_feat.parent_id)
                        self.ui.te_seq.verticalScrollBar().setValue(self.itemList[seq_feat.parent_id].memTWpos)


    """
    def alignOligo2(self):
        "align with bl2seq"
        i=self.checkSeq([oligo_item])
        if i<>None:
                       
            seq1=self.itemList[i].seq
            
            frm_show_results=ShowResults()
            sr=''

            all_blast_results=[]
            for j in self.itemList.keys():
                if self.itemList[j].type==sequence_item:
                    seq2=self.itemList[j].seq

                    blast_results=blast2seq(self.itemList[i].name,seq1,self.itemList[j].name,seq2)
                    if 'bl2seq error' in blast_results:
                        QMessageBox.warning(self,"pyAnnHyb",'bl2seq not found')
                        return
                    if blast_results=='no hits found':
                        sr=sr+'%s: %s\n\n' % (self.itemList[j].name,blast_results)
                    else:
                        all_blast_results+=blast_results
                        for r in blast_results:
                            sr=sr+'%s\nIdentity: %s\n%s\n%s\n%s\n\n' % (r['subject_name'],r['identity'],r['query'],r['match'],r['subject'])

            sr='Alignment results for '+self.itemList[i].name+'\n\n'+sr

            
            print all_blast_results
            frmAlignmentsList=AlignmentsList()
            frmAlignmentsList.ui.twAlignmentsList.setRowCount(len(all_blast_results))
            frmAlignmentsList.ui.twAlignmentsList.setSortingEnabled(False)
            
            display_pos,subject_pos,identity_pos,start_pos,end_pos,alignment_pos=0,1,2,3,4,5    
            for alignment in all_blast_results:
                
                    #checkbox inside QTabbleWidget
                    item=QTableWidgetItem('')
                    item.setFlags(Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsUserCheckable)
                    if float(alignment['identity'].replace('%',''))>=90:
                        item.setCheckState(Qt.Checked)
                    else:
                        item.setCheckState(Qt.Unchecked)
                        
                    frmAlignmentsList.ui.twAlignmentsList.setItem(all_blast_results.index(alignment),display_pos,item)

                    frmAlignmentsList.ui.twAlignmentsList.setItem(all_blast_results.index(alignment),subject_pos,QTableWidgetItem(alignment['subject_name']))
                    frmAlignmentsList.ui.twAlignmentsList.setItem(all_blast_results.index(alignment),identity_pos,QTableWidgetItem(alignment['identity']))
                    frmAlignmentsList.ui.twAlignmentsList.setItem(all_blast_results.index(alignment),start_pos,QTableWidgetItem(alignment['subject_start']))
                    frmAlignmentsList.ui.twAlignmentsList.setItem(all_blast_results.index(alignment),end_pos,QTableWidgetItem(alignment['subject_end']))
                    
                    #show alignment
                    item=QTableWidgetItem(alignment['query']+'\n'+alignment['match']+'\n'+alignment['subject'])
                    item.setFont(monospaceFont(10))
                    
                    frmAlignmentsList.ui.twAlignmentsList.setItem(all_blast_results.index(alignment),alignment_pos,item)
                    #frmAlignmentsList.ui.twAlignmentsList.resizeRowToContents(all_blast_results.index(alignment))            
                    frmAlignmentsList.ui.twAlignmentsList.setRowHeight(all_blast_results.index(alignment),80)
            frmAlignmentsList.ui.twAlignmentsList.resizeColumnsToContents()
            
            frmAlignmentsList.setWindowTitle(QString('Alignment results for '+self.itemList[i].name))
            frmAlignmentsList.exec_()
            
            #add checked alignments to annotations
            for row in range(0,frmAlignmentsList.ui.twAlignmentsList.rowCount()):  
                display=frmAlignmentsList.ui.twAlignmentsList.item(row,0)
                if display.checkState()==Qt.Checked:
                    seq_feat=Seq_Feat()
                    subject=str(frmAlignmentsList.ui.twAlignmentsList.item(row,subject_pos).text())
                    seq_feat.name=self.itemList[i].name  #oligo name
                    seq_feat.comment=str(seq_feat.name+' bl2seq alignment')
                    seq_feat.begin=int(frmAlignmentsList.ui.twAlignmentsList.item(row,start_pos).text())
                    seq_feat.end=int(frmAlignmentsList.ui.twAlignmentsList.item(row,end_pos).text())
                    identity=int(str(frmAlignmentsList.ui.twAlignmentsList.item(row,identity_pos).text()).replace('%',''))
                    #seq_feat.color=(255,int(-128.0/(100-90)*identity+128.0/(100-90)*90),int(-128.0/(100-90)*identity+128+128.0/(100-90)*90),255) #insert color gradient based on identity?
                    seq_feat.color=(255,0,0,255) #insert color gradient based on identity?
                    print seq_feat.color 
                    for k,item in self.itemList.iteritems():
                        if item.type==sequence_item:
                            #print item.name,subject
                            if item.name==subject:
                                #print item.name,subject,'OK'
                                self.itemList[k].features.append(seq_feat)

    """
    def annotationsList(self):
        """show annotations list window"""

        item = self.checkItem()

        if item['type'] in [sequence_item, seq_annot_item]:

            ### indicate sequence index in annotation selected
            if item['type'] in [seq_annot_item]:
                seq_id = self.itemList[item['index']].parent_id
            else:
                seq_id = item['index']


            print 'itemList', self.itemList

            while True:
                frmFeaturesList = FeaturesList()
                itemCount = 0

                ### count annotations
                for item2 in self.itemList:
                    if self.itemList[item2].type == seq_annot_item and self.itemList[item2].parent_id == seq_id:
                        itemCount += 1

                frmFeaturesList.ui.twFeaturesList.setRowCount(itemCount)
                frmFeaturesList.ui.twFeaturesList.setSortingEnabled(False)

                row = 0

                for item2 in self.itemList:
                    if self.itemList[item2].type == seq_annot_item and self.itemList[item2].parent_id == seq_id:
                        feature = self.itemList[item2]

                        #item.setTextAlignment(Qt.AlignCenter)
                        frmFeaturesList.ui.twFeaturesList.setItem(row, 0, QTableWidgetItem(str(feature.name)))
                        frmFeaturesList.ui.twFeaturesList.setItem(row, 1, QTableWidgetItem(str(feature.begin)))
                        frmFeaturesList.ui.twFeaturesList.setItem(row, 2, QTableWidgetItem(str(feature.end)))
                        frmFeaturesList.ui.twFeaturesList.setItem(row, 3, QTableWidgetItem(str(feature.comment)))

                        newItem = QTableWidgetItem()

                        dummyColor = QColor(feature.color[0], feature.color[1], feature.color[2], feature.color[3])
                        newItem.setBackgroundColor(dummyColor)

                        newItem.setData(Qt.UserRole, QVariant(item2))
                        print 'annot index', item2
                        frmFeaturesList.ui.twFeaturesList.setItem(row, 4, newItem)
                        row += 1

                self.ui.ta_oligoList.resizeColumnsToContents()

                frmFeaturesList.exec_()

                if frmFeaturesList.command == 'delete':

                    del self.itemList[int(frmFeaturesList.indexDict)]

                    self.ui.tv.removeItemWidget(tvItems[int(frmFeaturesList.indexDict)], 0)
                    del tvItems[int(frmFeaturesList.indexDict)]

                    self.insertAnnotations(seq_id)
                    self.projectChanged = True

                elif frmFeaturesList.command == 'changed':

                    feature = self.itemList[int(frmFeaturesList.indexDict)]
                    ### annotation name
                    if frmFeaturesList.index[1] == 0:
                        feature.name = frmFeaturesList.value
                        tvItems[int(frmFeaturesList.indexDict)].setText(0, feature.name)

                    ### annotation begin    
                    if frmFeaturesList.index[1] == 1:
                        if frmFeaturesList.value <= 0:
                            QMessageBox.warning(self, "pyAnnHyb", 'Begin value <=0 (%d)' % frmFeaturesList.value)
                        elif frmFeaturesList.value > len(self.itemList[seq_id].seq):
                            QMessageBox.warning(self, "pyAnnHyb", 'Begin value > sequence length (%d)' % len(self.itemList[seq_id].seq))
                        elif frmFeaturesList.value >= int(feature.end):
                            QMessageBox.warning(self, "pyAnnHyb", 'Begin value (%d) is greater than or equal to end value (%d):' % (frmFeaturesList.value, int(feature.end)))
                        else:
                            feature.begin = frmFeaturesList.value

                    ### annotation end        
                    if frmFeaturesList.index[1] == 2:
                        if frmFeaturesList.value <= 0:
                            QMessageBox.warning(self, "pyAnnHyb", 'End value <=0 (%d)' % frmFeaturesList.value)
                        elif frmFeaturesList.value > len(self.itemList[seq_id].seq):
                            QMessageBox.warning(self, "pyAnnHyb", 'End value > sequence length (%d)' % len(self.itemList[seq_id].seq))
                        elif frmFeaturesList.value <= int(feature.begin):
                            QMessageBox.warning(self, "pyAnnHyb", 'End value (%d) is less than or equal to begin value (%d):' % (frmFeaturesList.value, int(feature.begin)))
                        else:

                            feature.end = frmFeaturesList.value

                    ### annotation comment        
                    if frmFeaturesList.index[1] == 3:
                        feature.comment = frmFeaturesList.value
                    ### annotation color
                    if frmFeaturesList.index[1] == 4:
                        feature.color = frmFeaturesList.value
                        ### set background color for annotation
                        colorFeature = QColor(feature.color[0], feature.color[1], feature.color[2], feature.color[3])
                        print 'int(frmFeaturesList.indexDict):', int(frmFeaturesList.indexDict)
                        print tvItems
                        tvItems[int(frmFeaturesList.indexDict)].setBackgroundColor(0, colorFeature)

                    #update feature
                    self.itemList[int(frmFeaturesList.indexDict)] = feature

                    #self.itemList[item['index']].memFeatures={}
                    self.insertAnnotations(seq_id)

                    self.projectChanged = True

                else:
                    break
        else:
            QMessageBox.warning(self, 'pyAnnHyb', 'No sequence selected')

    def revertComplement(self):
        """revert complement selected sequence or oligo"""
        item = self.checkItem()

        if item['type'] in [sequence_item, oligo_item, seq_annot_item]:

            if item['type'] in [seq_annot_item]:
                i = self.itemList[item['index']].parent_id
            else:
                i = item['index']

            name = self.itemList[i].name

            seq_rev_comp = rev_comp_seq (convert2IUPAC(str(self.itemList[i].seq)).replace(' ', ''))
            seq_rev_comp = 'Revert complement of ' + name + '\n\n' + seq_rev_comp

            frm_show_results = ShowResults()
            frm_show_results.setWindowTitle(QString('Revert complement of ' + self.itemList[i].name))
            frm_show_results.ui.teResults.setText(QString(seq_rev_comp))
            frm_show_results.exec_()
        else:
            QMessageBox.warning(self, 'pyAnnHyb', 'No oligonucleotides or sequence selected')



    def translation(self):
        """translate selected sequence or oligo"""
        item = self.checkItem()

        if item['type'] in [sequence_item, oligo_item, seq_annot_item]:

            ### indicate sequence index in annotation selected
            if item['type'] in [seq_annot_item]:
                i = self.itemList[item['index']].parent_id
            else:
                i = item['index']

            seq_to_translate = convert2IUPAC(str(self.itemList[i].seq)).replace(' ', '')
            name = self.itemList[i].name

            #check if selection
            pos1 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionStart())
            pos2 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionEnd() - 1)
            selectedSeq = clear_seq(str(self.ui.te_seq.textCursor().selection().toPlainText()))

            if selectedSeq:
                if self.ynMessageBox('Do you want to translate only the selected sequence?', '') == 'Yes':
                    seq_to_translate = selectedSeq
                    name = '%s (from nt %d to %d)' % (self.itemList[i].name, pos1, pos2)

            dialog = translation_parameters()
            if dialog.exec_():
                frames = []
                if dialog.ui.cbFrame1.isChecked():
                    frames.append(1)
                if dialog.ui.cbFrame2.isChecked():
                    frames.append(2)
                if dialog.ui.cbFrame3.isChecked():
                    frames.append(3)
                if dialog.ui.cbFrame4.isChecked():
                    frames.append(4)
                if dialog.ui.cbFrame5.isChecked():
                    frames.append(5)
                if dialog.ui.cbFrame6.isChecked():
                    frames.append(6)
                if not frames:
                    QMessageBox.warning(self, "pyAnnHyb", 'You must choose at least one frame')
                    return None
                if dialog.ui.rb1letter.isChecked():
                    aa_code = 1
                if dialog.ui.rb3letters.isChecked():
                    aa_code = 3

                tr_seq = seq_translation(name, seq_to_translate, frames, 1, dialog.ui.cbIncludeNtSequence.isChecked())

                #biopython
                """
                from Bio.Seq import Seq
                from Bio.Alphabet.IUPAC import IUPACAmbiguousDNA
                tr_seq=Seq(str(self.itemList[i].seq),IUPACAmbiguousDNA()).translate()
                """
                #show results
                tr_seq = 'Translation of ' + name + '\n\n' + tr_seq
                frm_show_results = ShowResults()
                frm_show_results.setWindowTitle(QString('Translation of ' + name))
                frm_show_results.ui.teResults.setText(QString(str(tr_seq)))
                frm_show_results.exec_()
        else:
            QMessageBox.warning(self, 'pyAnnHyb', 'No oligonucleotides or sequence selected')

    def cropSequence(self):

        item = self.checkItem()

        if item['type'] in [sequence_item, oligo_item, seq_annot_item]:

            ### indicate sequence index in annotation selected
            if item['type'] in [seq_annot_item]:
                i = self.itemList[item['index']].parent_id
            else:
                i = item['index']

            seq = convert2IUPAC(str(self.itemList[i].seq)).replace(' ', '')
            name = self.itemList[i].name

            ### check if selection
            pos1 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionStart())
            pos2 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionEnd() - 1)
            selectedSeq = clear_seq(str(self.ui.te_seq.textCursor().selection().toPlainText()))

            if selectedSeq:
                if self.ynMessageBox('Do you want to crop the selected sequence?', '') == 'Yes':
                    self.itemList[i].seq = selectedSeq

                    ### check annotations
                    mem_to_del = []
                    for index in self.itemList:
                        if self.itemList[index].type == seq_annot_item and self.itemList[index].parent_id == i:

                            ### annotation included in crop seq
                            if self.itemList[index].begin >= pos1 and self.itemList[index].end <= pos2:
                                self.itemList[index].begin = self.itemList[index].begin - pos1 + 1
                                self.itemList[index].end = self.itemList[index].end - pos1 + 1

                            elif  self.itemList[index].begin >= pos1 and self.itemList[index].begin <= pos2 and  self.itemList[index].end > pos2:
                                self.itemList[index].begin = self.itemList[index].begin - pos1 + 1
                                self.itemList[index].end = len(selectedSeq)

                            elif  self.itemList[index].begin < pos1 and  self.itemList[index].end <= pos2 and self.itemList[index].end >= pos1:
                                self.itemList[index].begin = 1
                                self.itemList[index].end = self.itemList[index].end - pos1 + 1

                            elif  self.itemList[index].begin < pos1 and  self.itemList[index].end > pos2:
                                self.itemList[index].begin = 1
                                self.itemList[index].end = len(selectedSeq)

                            else:
                                mem_to_del.append(index)


                    print 'mem to del', mem_to_del
                    ### delete annotations out of cropped sequence    
                    for ad in mem_to_del:
                        del self.itemList[ad]
                        self.ui.tv.removeItemWidget(tvItems[ad], 0)
                        del tvItems[ad]
                    ### update sequence
                    self.insertAnnotations(i)
                    self.projectChanged = True
            else:
                QMessageBox.warning(self, "pyAnnHyb", 'No sequence selection to crop...')


    def deleteSequence(self):
        """delete sequence selection"""
        print 'delete'
        item = self.checkItem()

        if item['type'] in [sequence_item, oligo_item, seq_annot_item]:

            ### indicate sequence index in annotation selected
            if item['type'] in [seq_annot_item]:
                i = self.itemList[item['index']].parent_id
            else:
                i = item['index']

            seq = convert2IUPAC(str(self.itemList[i].seq)).replace(' ', '')
            name = self.itemList[i].name

            ### check if selection
            pos1 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionStart())
            pos2 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionEnd() - 1)
            selectedSeq = clear_seq(str(self.ui.te_seq.textCursor().selection().toPlainText()))

            if selectedSeq:
                if self.ynMessageBox('Do you want to delete the selected sequence?', '') == 'Yes':
                    ###
                    #print self.itemList[i].seq[pos1:pos2]
                    self.itemList[i].seq = self.itemList[i].seq[0:pos1 - 1] + self.itemList[i].seq[pos2:]
                    ### check annotations
                    mem_to_del = []
                    for index in self.itemList:
                        if self.itemList[index].type == seq_annot_item and self.itemList[index].parent_id == i:

                            ### annotation after deleted seq
                            if self.itemList[index].begin > pos2:
                                self.itemList[index].begin = self.itemList[index].begin - (pos2 - pos1 + 1)
                                self.itemList[index].end = self.itemList[index].end - (pos2 - pos1 + 1)

                            ### annotation in deleted seq
                            if self.itemList[index].begin >= pos1 and self.itemList[index].begin <= pos2 \
                               and self.itemList[index].end >= pos1 and self.itemList[index].end <= pos2:
                                mem_to_del.append(index)

                            ### delete region over annotation init
                            if self.itemList[index].begin >= pos1 and self.itemList[index].begin <= pos2 and self.itemList[index].end >= pos2:
                                self.itemList[index].begin = pos2 + 1 - (pos2 - pos1 + 1)
                                self.itemList[index].end = self.itemList[index].end - (pos2 - pos1 + 1)
                                ###self.itemList[index].begin = pos2 + 1

                            ### delete region over annotation end
                            if self.itemList[index].begin < pos1 and self.itemList[index].end > pos1 and self.itemList[index].end < pos2:
                                self.itemList[index].end = pos1 - 1




                    print 'mem to del', mem_to_del
                    ### delete annotations in deleted sequence    
                    for ad in mem_to_del:
                        del self.itemList[ad]
                        self.ui.tv.removeItemWidget(tvItems[ad], 0)
                        del tvItems[ad]
                    ### update sequence
                    self.insertAnnotations(i)
                    self.projectChanged = True

                    self.ui.le_seqLength.setText(str(oligo_length(self.itemList[i].seq)))
            else:
                QMessageBox.warning(self, "pyAnnHyb", 'No sequence selection to delete...')



    def saveSequence(self):
        """Save DNA sequence"""
        item = self.checkItem()
        if item['type'] == sequence_item:

            fd1 = QFileDialog(self)
            #fd1.setFilters(['FASTA (*.fasta)','GenBank (*.gb)','EMBL (*.embl)'])
            #print 'filter:',str(fd1.selectedFilter())

            filename = str(fd1.getSaveFileName(self, 'Save the current DNA sequence', '', 'All files (*)'))
            #print 'filter:',fd1.filter()
            if filename:
            #set sequence format
                seq_format = ''
                if filename.find('.fasta') <> -1:
                    seq_format = 'fasta'
                if filename.find('.gb') <> -1:
                    seq_format = 'genbank'
                if filename.find('.html') <> -1:
                    seq_format = 'html'

                #if filename.find('.embl')<>-1:
                #    seq_format='embl'

                if not seq_format:
                   QMessageBox.warning(self, "pyAnnHyb", 'Sequence format not recognized\nUse .fasta or .gb as file extension.')
                   return 1

                f = open(filename, 'w')
                i = item['index']
                if seq_format == 'html':
                    f.write('<b>' + self.itemList[i].name + '</b><br>')
                    if self.itemList[i].description:
                        f.write(self.itemList[i].description + '<br>')

                    f.write('<br>')
                    f.write(str(self.ui.te_seq.toHtml()))

                    for k in self.itemList:
                        if self.itemList[k].type == seq_annot_item and self.itemList[k].parent_id == i:
                            feature = self.itemList[k]

                            hexcolor = '#%02x%02x%02x' % self.itemList[k].color[:-1]

                            f.write('<span style=" background-color:%s;">%s</span> %s-%s<br>%s<br><br>' % (hexcolor, str(feature.name), str(feature.begin), str(feature.end), str(feature.comment)))
                else:

                    from Bio import SeqIO
                    from Bio.SeqRecord import SeqRecord
                    from Bio.Seq import Seq
                    from Bio.Alphabet import generic_dna, generic_protein

                    i = int(self.ui.tv.selectedItems()[0].text(1))
                    record = SeqRecord(Seq(self.itemList[i].seq, generic_dna),
                               id=self.itemList[i].name, name=self.itemList[i].name,
                               description=self.itemList[i].description)

                    SeqIO.write([record], f, seq_format)
                f.close()
        else:
            QMessageBox.warning(self, 'pyAnnHyb', 'No selected sequence to save')

    def openSequence(self):
        """load DNA sequence from file"""
        fd = QFileDialog(self)
        newfile = fd.getOpenFileName(self, "Open a DNA sequence", "", "All files (*)")
        if os.path.isfile(newfile):
            #recognize sequence format
            seq_format = None
            first_row = open(newfile, "rU").readline()
            if first_row.find('>') == 0:
                seq_format = 'fasta'
            if first_row.find('LOCUS ') == 0:
                seq_format = 'genbank'
            if first_row.find('ID ') == 0:
                seq_format = 'embl'
            if first_row.find('CLUSTAL') == 0:
                seq_format = 'clustal'

            if not seq_format:
                QMessageBox.warning(self, "pyAnnHyb", 'Sequence format not recognized')
                return 1

            f = open(newfile, "rU")
            for record in SeqIO.parse(f, seq_format) :
                s = Sequence()
                s.name = record.name
                #s.longName=seq.findtext('sequence_longname')
                s.description = record.description
                #s.type_=

                #s.author=seq.findtext('author')
                #s.creation_date=parsedSeq.date
                #s.header=seq.findtext('sequence_header')
                #s.features=[]                 #parsedSeq.features

                s.seq = clear_seq(str(record.seq))

                id = len(self.itemList) + 1
                self.itemList[id] = s
                #seqitem = QTreeWidgetItem(self.seq_items,[self.itemList[id].name,str(id),str(sequence_item)]) #set seq name and index

                seqitem = QTreeWidgetItem(catList['seqList']['item'], [self.itemList[id].name, str(id), str(sequence_item)]) #set seq name and index

                tvItems[id] = seqitem

                seqitem.setSelected(True)
                self.projectChanged = True

            f.close()


    def le_seqName_changed(self):
        """Seq name changed"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        if self.itemList.has_key(i):

            ### check for  name
            flag = False
            for j in self.itemList.keys():
                if j != i and self.itemList[j].type == sequence_item and self.itemList[j].name == self.ui.le_seqName.text():
                    flag = True

            if flag:
                QMessageBox.warning(self, "pyAnnHyb", 'Sequence <b>' + self.ui.le_seqName.text() + '</b> already exists!')
            else:

                self.itemList[i].name = self.ui.le_seqName.text()
                self.projectChanged = True
                self.ui.tv.selectedItems()[0].setText(0, str(self.ui.le_seqName.text()))

    def le_seqAuthor_changed(self):
        """Seq author changed"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        if self.itemList.has_key(i):
            self.itemList[i].author = self.ui.le_seqAuthor.text()
            self.projectChanged = True

    def le_longName_changed(self):
        """Seq long name changed"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        if self.itemList.has_key(i):
            self.itemList[i].longName = self.ui.le_longName.text()
            self.projectChanged = True

    def le_seqDefinition_changed(self):
        """Seq definition changed"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        if self.itemList.has_key(i):
            self.itemList[i].description = self.ui.le_seqDefinition.text()
            self.projectChanged = True

    def le_seqDate_changed(self):
        """Seq date changed"""
        i = int(self.ui.tv.selectedItems()[0].text(1))
        if self.itemList.has_key(i):
            self.itemList[i].creation_date = self.ui.le_seqDate.text()
            self.projectChanged = True

    def ynMessageBox(self, MessageText, DetailedText):
        response = False
        YES = 'Yes'
        NO = 'No'
        message = QMessageBox(self)
        message.setText(MessageText)
        message.setWindowTitle('pyAnnHyb')
        message.setIcon(QMessageBox.Question)
        message.addButton(YES, QMessageBox.AcceptRole)
        message.addButton(NO, QMessageBox.DestructiveRole)
        message.setDetailedText(DetailedText)
        message.exec_()
        response = message.clickedButton().text()
        return response

    def closeEvent(self, event):
        """Save configuration, check if project saved and close program"""
        self.saveConfigFile()
        response = ''
        CANCEL = 'Cancel'
        if self.projectChanged:
            response = False
            SAVE = 'Save'
            DISCARD = 'Discard'
            message = QMessageBox(self)
            message.setText('What to do about project unsaved changes?')
            message.setWindowTitle('pyAnnHyb')
            message.setIcon(QMessageBox.Question)
            message.addButton(SAVE, QMessageBox.AcceptRole)
            message.addButton(DISCARD, QMessageBox.DestructiveRole)
            message.addButton(CANCEL, QMessageBox.RejectRole)
            message.setDetailedText('Unsaved changes in project: ' + str(self.projectFileName))
            message.exec_()
            response = message.clickedButton().text()
            if response == SAVE:
                self.saveProject()

        if response == CANCEL:
            event.ignore()

    def readConfigFile(self):
        filename = os.path.expanduser('~')

        if platform.system() == 'Linux':
            filename += '/'
        elif platform.system() == 'Windows':
            filename += '\\'
        else:
            filename = ''
        filename += '.pyannhyb'
        if os.path.isfile(filename):
            ini = QSettings(filename, QSettings.IniFormat)
            size = ini.value("MainWindow/Size").toSize()
            self.resize(size)
            position = ini.value("MainWindow/Position").toPoint()
            self.move(position)
            self.restoreState(ini.value("MainWindow/State").toByteArray())

            pref_keys = self.preferences.keys()
            for k in pref_keys:
                try:
                    self.preferences[k] = ini.value(k).toString()
                except:
                    pass

    def saveConfigFile(self):
        filename = os.path.expanduser('~')
        if platform.system() == 'Linux':
            filename = filename + '/'
        elif platform.system() == 'Windows':
            filename = filename + '\\'
        else:
            filename = ''
        filename = filename + '.pyannhyb'
        ini = QSettings(filename, QSettings.IniFormat)

        ini.setValue("MainWindow/State", QVariant(self.saveState()))
        ini.setValue("MainWindow/Size", QVariant(self.size()))
        ini.setValue("MainWindow/Position", QVariant(self.pos()))

        for k, v in self.preferences.iteritems():
            ini.setValue(k, QVariant(v))

    def setPreferences(self):
        """Set program preferences"""

        frm_preferences = Preferences()
        #print frm_preferences.ui.tw_mec.item(0,0).text()
        #MEC
        for i, v in enumerate(['mecA', 'mecC', 'mecG', 'mecT']):
            frm_preferences.ui.tw_mec.item(0, i).setText(str(self.preferences[v]))
        #TM

        frm_preferences.ui.le_oligoDefaultSaltConc.setText(str(self.preferences['oligoDefaultSaltConc']))
        frm_preferences.ui.le_oligoDefaultConc.setText(str(self.preferences['oligoDefaultConc']))
        frm_preferences.ui.le_seqSelectionDefaultSaltConc.setText(str(self.preferences['seqSelectionDefaultSaltConc']))
        frm_preferences.ui.le_seqSelectionDefaultConc.setText(str(self.preferences['seqSelectionDefaultConc']))
        frm_preferences.ui.le_sequenceDefaultSaltConc.setText(str(self.preferences['sequenceDefaultSaltConc']))
        frm_preferences.ui.leCheckAlignmentCutOff.setText(str(self.preferences['CheckAlignmentCutOff']))
        ### checksum
        if self.preferences['checksum'] == 'SEGUID':
            frm_preferences.ui.rbSEGUIDChecksum.setChecked(True)
        if self.preferences['checksum'] == 'GCG':
            frm_preferences.ui.rbGCGChecksum.setChecked(True)

        frm_preferences.exec_()

        if frm_preferences.result():
            for i, v in enumerate(['mecA', 'mecC', 'mecG', 'mecT']):
                self.preferences[v] = str(frm_preferences.ui.tw_mec.item(0, i).text())

            self.preferences['oligoDefaultSaltConc'] = str(frm_preferences.ui.le_oligoDefaultSaltConc.text())
            self.preferences['oligoDefaultConc'] = str(frm_preferences.ui.le_oligoDefaultConc.text())
            self.preferences['seqSelectionDefaultSaltConc'] = str(frm_preferences.ui.le_seqSelectionDefaultSaltConc.text())
            self.preferences['seqSelectionDefaultConc'] = str(frm_preferences.ui.le_seqSelectionDefaultConc.text())
            self.preferences['sequenceDefaultSaltConc'] = str(frm_preferences.ui.le_sequenceDefaultSaltConc.text())
            self.preferences['CheckAlignmentCutOff'] = str(frm_preferences.ui.leCheckAlignmentCutOff.text())

            if frm_preferences.ui.rbSEGUIDChecksum.isChecked():
                self.preferences['checksum'] = 'SEGUID'
            if frm_preferences.ui.rbGCGChecksum.isChecked():
                self.preferences['checksum'] = 'GCG'

            ### update selected oligo parameters
            item = self.checkItem()
            if item['type'] in [oligo_item]:
                self.oligoParamUpdate(i)
            if item['type'] in [oligoList]:
                self.fillOligoList()

            if item['type'] in [seqList]:
                self.fillSeqList()


        self.saveConfigFile()
        #self.fillOligoList()

    def newSeqAnnotation(self):
        """add new annotation to selected sequence"""

        item = self.checkItem()
        #print 'item',item

        if item['type'] in [sequence_item, seq_annot_item]:

            ### indicate sequence index in annotation selected
            if item['type'] in [seq_annot_item]:
                i = self.itemList[item['index']].parent_id
            else:
                i = item['index']

            pos1 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionStart())
            pos2 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionEnd() - 1)

            u = unicode(self.ui.te_seq.textCursor().selectedText()).encode('utf-8')
            pt = str(self.ui.te_seq.textCursor().selection().toPlainText())

            frm_seq_feature = SeqFeature()
            frm_seq_feature.ui.leStart.setText(str(pos1))
            frm_seq_feature.ui.leEnd.setText(str(pos2))

            def drawPixmap(w, h, color):
                pixmap = QPixmap(w, h)
                painter = QPainter(pixmap)
                painter.fillRect(pixmap.rect(), color)
                return pixmap

            frm_seq_feature.ui.lbColor.setPixmap(drawPixmap(100, 30, QColor('yellow')))

            if frm_seq_feature.exec_():  #button OK

                pos1 = int(frm_seq_feature.ui.leStart.text())
                pos2 = int(frm_seq_feature.ui.leEnd.text())
                if pos1 == -1:
                    QMessageBox.warning(self, 'pyAnnHyb', 'Input a valid range to annotate')
                    return 0
                if pos1 > pos2:
                    QMessageBox.warning(self, 'pyAnnHyb', 'Start position is greater than end position')
                    return 0

                self.itemList[i].formatted_seq = QString(self.ui.te_seq.toHtml())
                #print self.itemList[int(self.ui.tv.selectedItems()[0].text(1))].formatted_seq

                ### create new annotation Class
                seq_feat = Seq_Feat()
                seq_feat.name = str(frm_seq_feature.ui.leName.text())
                seq_feat.comment = str(frm_seq_feature.ui.leComment.text())
                seq_feat.begin = pos1
                seq_feat.end = pos2
                seq_feat.color = frm_seq_feature.color.getRgb()  #tuple
                seq_feat.parent_id = i

                print 'item', item
                print 'tvItems', tvItems
                print 'tvItems[item[index]]', tvItems[i]

                ### new item_id for feature
                item_id = len(self.itemList) + 1

                self.itemList[item_id] = seq_feat


                newFeature = QTreeWidgetItem(
                                             tvItems[i],
                                             [seq_feat.name, str(item_id), str(seqAnnotation)]

                                             )
                tvItems[item_id] = newFeature

                self.ui.tv.expandItem(tvItems[i])
                newFeature.setSelected(True)
                tvItems[i].setSelected(False)

                ### set background color for annotation
                colorFeature = QColor(seq_feat.color[0], seq_feat.color[1], seq_feat.color[2], seq_feat.color[3])
                newFeature.setBackgroundColor(0, colorFeature)



                self.itemList[i].memFeatures = []
                self.projectChanged = True

                self.insertAnnotations(i)
                self.ui.te_seq.verticalScrollBar().setValue(self.itemList[i].memTWpos)


    def restrictionAnalysis(self):
        """restriction analysis of selected sequence (oligo or sequence) with biopython package"""
        item = self.checkItem()
        if item['type'] in [sequence_item, oligo_item]:
            i = item['index']
            seq_to_cut = convert2IUPAC(str(self.itemList[i].seq)).replace(' ', '')
            name = self.itemList[i].name

            #check if selection
            pos1 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionStart())
            pos2 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionEnd() - 1)
            selectedSeq = clear_seq(str(self.ui.te_seq.textCursor().selection().toPlainText()))

            if selectedSeq:
                if self.ynMessageBox('Do you want to analyze only the selected sequence?', '') == 'Yes':
                    seq_to_cut = selectedSeq
                    name = '%s (from nt %d to %d)' % (self.itemList[i].name, pos1, pos2)

            frm_re_selection = RESelection()
            frm_re_selection.ui.lwAvailableRE.clear()
            frm_re_selection.ui.lwSelectedRE.clear()

            for re in AllEnzymes:
                frm_re_selection.ui.lwAvailableRE.addItem(str(re))
            frm_re_selection.ui.lwAvailableRE.sortItems()
            if frm_re_selection.exec_():
                rb = RestrictionBatch([])
                for row in range(frm_re_selection.ui.lwSelectedRE.count()):
                    rb.add(str(frm_re_selection.ui.lwSelectedRE.item(row).text()))

                if rb:
                    re_seq = Seq(str(seq_to_cut), IUPACAmbiguousDNA())
                    re_analysis = Analysis(rb, re_seq, linear=True)

                    #results format
                    print frm_re_selection.ui.rbList.isChecked()
                    print frm_re_selection.ui.rbMap.isChecked()
                    if frm_re_selection.ui.rbList.isChecked():
                        #print 'list format'
                        re_full = re_analysis.full()
                    if frm_re_selection.ui.rbMap.isChecked():
                        #print 'map format'                        
                        re_full = re_analysis.print_as('map')


                    mem_stdout = sys.stdout
                    import tempfile
                    temp_fn = tempfile.mktemp()

                    sys.stdout = open(temp_fn, 'w')

                    re_analysis.print_that()
                    sys.stdout.close()

                    sys.stdout = mem_stdout

                    text_result = open(temp_fn, 'r').read()

                    text_result = 'Restriction analysis of ' + name + '\n\n' + text_result

                    frm_show_results = ShowResults()
                    frm_show_results.setWindowTitle(QString('Restriction analysis of ' + name))
                    frm_show_results.ui.teResults.setText(QString(text_result))
                    frm_show_results.exec_()
                else:
                    QMessageBox.warning(self, 'pyAnnHyb', 'No restriction enzymes selected')
        else:
            QMessageBox.warning(self, 'pyAnnHyb', 'No oligonucleotides or sequence selected')

    def retrieveSequence(self):
        """retrieve sequence from NCBI server by AC"""
        ask_USA = frmAskValue()
        ask_USA.setWindowTitle('Retrieve sequence from NCBI server')
        ask_USA.ui.label.setText('Sequence identifier')
        if ask_USA.exec_():  #button OK
            USA = str(ask_USA.ui.lineEdit.text()).strip()
            if not USA:
                QMessageBox.warning(self, "pyAnnHyb", 'You do not specify any sequence identifier')
                return 3

            import urllib2
            url = 'http://eutils.ncbi.nlm.nih.gov/entrez/eutils/efetch.fcgi?db=nucleotide&id=*&rettype=gb'
            url = url.replace('*', USA)
            remoteSeq = urllib2.urlopen(url).read()

            if len(remoteSeq) == 1:
                QMessageBox.warning(self, "pyAnnHyb", '<b>%s</b> is not a valid sequence identifier' % USA)
                return 1

            import tempfile
            tempFileName = tempfile.mktemp()
            f = open(tempFileName, 'w').write(remoteSeq)

            parser = GenBank.RecordParser()
            parsedSeq = parser.parse(open(tempFileName))

            if not parsedSeq:
                QMessageBox.warning(self, "pyAnnHyb", '<b>%s</b> is not a valid sequence identifier' % USA)
                return 1

            s = Sequence()
            s.name = str(parsedSeq.accession[0])
            #s.longName=seq.findtext('sequence_longname')
            s.description = parsedSeq.definition
            s.type_ = parsedSeq.residue_type.split(' ')[0]

            #s.author=seq.findtext('author')
            s.creation_date = parsedSeq.date
            #s.header=seq.findtext('sequence_header')
            ##########################################
            ##########################################
            s.features = []                 #parsedSeq.features

            s.seq = str(parsedSeq.sequence)
    #        print s.seq
            id = len(self.itemList) + 1
            self.itemList[id] = s
            seqitem = QTreeWidgetItem(catList['seqList']['item'], [self.itemList[id].name, str(id), str(sequence_item)]) #set seq name, index and type

            tvItems[id] = seqitem

            ### if selected item deselect it
            if self.ui.tv.selectedItems():
                self.ui.tv.setItemSelected(self.ui.tv.selectedItems()[0], False)

            ### select retrieved sequence
            self.ui.tv.setItemSelected(seqitem, True)

            self.projectChanged = True

    def te_seq_selectionChanged(self):
        """sequence selection changed"""
        pos1 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionStart())
        pos2 = TextEditPos2SeqPos(self.ui.te_seq.textCursor().selectionEnd() - 1)

        #control
        """
        print 'selection start:',self.ui.te_seq.textCursor().selectionStart()
        print 'sequence pos:',pos1
        print 'seq 2 Textedit',seqPostoTextEditPos(pos1)
        print
        """
#        qs=QString(self.ui.te_seq.textCursor().selectedText())
#        for c in [' ','\xa0',u'\u2028','0','1','2','3','4','5','6','7','8','9']:
#            qs=qs.replace(c, '')
        s = clear_seq(str(self.ui.te_seq.textCursor().selection().toPlainText()))

        if pos1 < pos2:
            tm = nn_melting(s, float(self.preferences['seqSelectionDefaultSaltConc']), float(self.preferences['seqSelectionDefaultConc']))[0]
            if type(tm) != type(''):
                self.statusBar().showMessage('Selection: %d - %d (%d bp) Tm=%s�C' % (pos1, pos2, pos2 - pos1 + 1, tm), 0)
            else:
                self.statusBar().showMessage('Selection: %d - %d (%d bp) %s' % (pos1, pos2, pos2 - pos1 + 1, tm), 0)
        elif pos1 != -1:
            self.statusBar().showMessage('Position: %d' % (pos1), 0)
        else:
            self.statusBar().clearMessage()

    def oligoParamUpdate(self, i):
        """oligonucleotide parameters"""
        #oligo length
        self.ui.le_oligo_len.setText(str(oligo_length(self.itemList[i].seq)))
        #IUPAC oligo seq
        self.ui.le_IUPAC_seq.setText(convert2IUPAC(str(self.itemList[i].seq)))
        #GC percent
        self.ui.le_gc.setText(str(GCp(str(self.itemList[i].seq)) * 100))
        ### MW
        self.ui.leMW.setText(str(MW(self.itemList[i].seq)))
        ### Checksum
        self.ui.leChecksum.setText(seqChecksum(str(self.itemList[i].seq), self.preferences['checksum']))

        ### MEC
        self.ui.leMEC.setText(str(MEC(self.itemList[i].seq, int(self.preferences['mecA']), int(self.preferences['mecC']), int(self.preferences['mecG']), int(self.preferences['mecT']))))

        #rev-comp oligo seq
        self.ui.le_revcomp.setText(rev_comp_seq(str(self.itemList[i].seq)))
        #melting temperature
#        if int(self.ui.le_oligo_len.text())<8:
#            self.ui.le_melting_temp.setText('Oligo too short')
        if str(self.itemList[i].salt_cc) == '' or str(self.itemList[i].oligo_cc) == '':
            self.ui.le_melting_temp.setText('')
        else:
            self.ui.le_melting_temp.setText(str(nn_melting(str(self.itemList[i].seq), float(self.itemList[i].salt_cc), float(self.itemList[i].oligo_cc))[0]))

    def oligoName_changed(self):
        """Oligo name changed"""

        i = int(self.ui.tv.selectedItems()[0].text(1))
        if self.itemList.has_key(i):

            ### check for  name
            flag = False
            for j in self.itemList.keys():
                if j != i and self.itemList[j].type == oligo_item and self.itemList[j].name == self.ui.le_OligoName.text():
                    flag = True

            if flag:
                QMessageBox.warning(self, "pyAnnHyb", 'Oligo <b>' + self.ui.le_OligoName.text() + '</b> already exists!')
            else:
                self.itemList[i].name = self.ui.le_OligoName.text()
                self.projectChanged = True
                self.ui.tv.selectedItems()[0].setText(0, str(self.ui.le_OligoName.text()))

    def fillOligoList(self):
        """fill oligo list with all oligo"""

        count = 0
        for i in self.itemList.keys():
            if self.itemList[i].type == oligo_item:
                count = count + 1

        self.ui.ta_oligoList.setRowCount(count)
        self.ui.ta_oligoList.setSortingEnabled(False)
        row = 0
        for i in self.itemList.keys():
            if self.itemList[i].type == oligo_item:
                item = QTableWidgetItem(str(self.itemList[i].name))
                #item.setData(Qt.UserRole, QVariant('aaabbb'))
                #item.setTextAlignment(Qt.AlignCenter)
                self.ui.ta_oligoList.setItem(row, 0, item)

                item = QTableWidgetItem(str(self.itemList[i].seq))
                self.ui.ta_oligoList.setItem(row, 1, item)

                ### oligo length
                item = QTableWidgetItem(str(oligo_length(self.itemList[i].seq)))
                self.ui.ta_oligoList.setItem(row, 2, item)

                #item=QTableWidgetItem(str(GCG_checksum(self.itemList[i].seq)))
                #self.ui.ta_oligoList.setItem(row,3,item)

                ### oligo checksum
                item = QTableWidgetItem(seqChecksum(str(self.itemList[i].seq), self.preferences['checksum']))

                self.ui.ta_oligoList.setItem(row, 3, item)


                ### oligo MW
                item = QTableWidgetItem(str(MW(self.itemList[i].seq)))
                self.ui.ta_oligoList.setItem(row, 4, item)

                item = QTableWidgetItem(str(GCp(self.itemList[i].seq) * 100))
                self.ui.ta_oligoList.setItem(row, 5, item)

                item = QTableWidgetItem(str(MEC(self.itemList[i].seq, int(self.preferences['mecA']), \
                int(self.preferences['mecC']), \
                int(self.preferences['mecG']), \
                int(self.preferences['mecT']))))

                self.ui.ta_oligoList.setItem(row, 6, item)

                item = QTableWidgetItem(str(self.itemList[i].salt_cc))
                self.ui.ta_oligoList.setItem(row, 7, item)

                item = QTableWidgetItem(str(self.itemList[i].oligo_cc))
                self.ui.ta_oligoList.setItem(row, 8, item)

                item = QTableWidgetItem(str(nn_melting(self.itemList[i].seq, float(self.itemList[i].salt_cc), float(self.itemList[i].oligo_cc))[0]))
                self.ui.ta_oligoList.setItem(row, 9, item)
                row += 1
        self.ui.ta_oligoList.resizeColumnsToContents()

    def fillSeqList(self):
        """fill list of sequences"""
        count = 0
        for i in self.itemList.keys():
            if self.itemList[i].type == sequence_item:
                count += 1

        self.ui.ta_seqList.setRowCount(count)
        self.ui.ta_seqList.setSortingEnabled(False)
        row = 0
        for k, item in self.itemList.iteritems():
            if item.type == sequence_item:
                table_item = QTableWidgetItem(str(item.name))
                #item.setData(Qt.UserRole, QVariant('aaabbb'))
                #item.setTextAlignment(Qt.AlignCenter)
                self.ui.ta_seqList.setItem(row, 0, table_item)

                table_item = QTableWidgetItem(str(oligo_length(item.seq)))
                self.ui.ta_seqList.setItem(row, 1, table_item)

                table_item = QTableWidgetItem(str(item.longName))
                self.ui.ta_seqList.setItem(row, 2, table_item)

                #table_item=QTableWidgetItem(str(GCG_checksum(item.seq)))
                #self.ui.ta_seqList.setItem(row,3,table_item)

                table_item = QTableWidgetItem(str(seqChecksum(item.seq, self.preferences['checksum'])))
                self.ui.ta_seqList.setItem(row, 3, table_item)


                table_item = QTableWidgetItem(str(item.description))
                self.ui.ta_seqList.setItem(row, 4, table_item)
                row += 1
        self.ui.ta_seqList.resizeColumnsToContents()

    def fillMAList(self):
        """fill list with multiple alignment"""
        count = 0
        for i in self.itemList.keys():
            if self.itemList[i].type == MSA_item:
                count += 1

        self.ui.ta_maList.setRowCount(count)
        self.ui.ta_maList.setSortingEnabled(False)
        row = 0
        for k, item in self.itemList.iteritems():
            if item.type == MSA_item:
                table_item = QTableWidgetItem(str(item.name))
                self.ui.ta_maList.setItem(row, 0, table_item)

                #seq length
                lenList = [len(s) for s in item.sequences.strip().split('\n')]
                minLen, maxLen = min(lenList), max(lenList)
                if minLen == maxLen:
                    table_item = QTableWidgetItem('%d' % minLen)
                else:
                    table_item = QTableWidgetItem('%d - %d' % (minLen, maxLen))
                self.ui.ta_maList.setItem(row, 1, table_item)

                #seq number
                #print item.sequences_names.strip().split('\n')
                table_item = QTableWidgetItem(str(len(item.sequences_names.strip().split('\n'))))
                self.ui.ta_maList.setItem(row, 2, table_item)
                row = row + 1
        self.ui.ta_maList.resizeColumnsToContents()

    def insertAnnotations(self, i):
        """insert sequence annotations in QTextView"""
        ### test if features changed

        if True:  ### self.itemList[i].features!=self.itemList[i].memFeatures:

            ### initialize formatted sequence
            self.itemList[i].formatted_seq = seq_format(self.itemList[i].seq)
            self.ui.te_seq.setHtml(QString(self.itemList[i].formatted_seq))

            for index in self.itemList:
                if self.itemList[index].type == seq_annot_item and self.itemList[index].parent_id == i:

                    """
                    for featureIndex in self.itemList[i].featuresDict:
                    feat=self.itemList[i].featuresDict[featureIndex]
                    init,end=seqPostoTextEditPos(int(feat.begin)),seqPostoTextEditPos(int(feat.end))
                    """

                    init, end = seqPostoTextEditPos(int(self.itemList[index].begin)), seqPostoTextEditPos(int(self.itemList[index].end))


                    cursor = self.ui.te_seq.textCursor()
                    cursor.setPosition(init)

                    cursor.setPosition(end + 1, QTextCursor.KeepAnchor)

                    hexcolor = '#%02x%02x%02x' % self.itemList[index].color[:-1]

                    pt = str(cursor.selection().toPlainText())
                    pt = str(pt)

                    pt = pt.replace(' ', '&nbsp;').replace('\n', '<br/>')
                    pt = '<span STYLE="background: %s">%s</span>' % (hexcolor, pt)
                    #print 'plain text selection',pt

                    cursor.insertHtml(QString(pt))

                    self.itemList[i].formatted_seq = QString(self.ui.te_seq.toHtml())

            self.ui.te_seq.setHtml(QString(self.itemList[i].formatted_seq))
            ### save current features
            """TODO: to be fixed"""
            #self.itemList[i].memFeatures=self.itemList[i].featuresDict
        else:
            if not self.itemList[i].formatted_seq:
                self.itemList[i].formatted_seq = seq_format(self.itemList[i].seq)
            self.ui.te_seq.setHtml(QString(self.itemList[i].formatted_seq))

        self.ui.te_seq.verticalScrollBar().setValue(self.itemList[i].memTWpos)

    def tv_selection_changed(self):
        """QTreeWidget selection changed"""
        item = self.checkItem()   ### dict with type and index
        print item
        #print tvItems

        if item['type'] == oligoList:
            self.fillOligoList()
            self.ui.tabWidget.setCurrentIndex(tabIndex['oligoList'])

        if item['type'] == seqList:
            self.fillSeqList()
            self.ui.tabWidget.setCurrentIndex(tabIndex['seqList'])

        if item['type'] == MAList:
            self.fillMAList()
            self.ui.tabWidget.setCurrentIndex(tabIndex['MAList'])

        if item['type'] == oligo_item:
            self.ui.le_oligo_seq.setText(self.itemList[item['index']].seq)
            self.ui.le_salt_cc.setText(str(self.itemList[item['index']].salt_cc / 1e-3))
            self.ui.le_oligo_cc.setText(str(self.itemList[item['index']].oligo_cc / 1e-9))
            self.ui.le_OligoName.setText(self.itemList[item['index']].name)
            self.oligoParamUpdate(item['index'])
            self.ui.tabWidget.setCurrentIndex(tabIndex['oligo'])

        if item['type'] in [sequence_item]:

            self.ui.le_seqName.setText(self.itemList[item['index']].name)

            if self.itemList[item['index']].longName:
                self.ui.le_longName.setText(self.itemList[item['index']].longName)

            if self.itemList[item['index']].author:
                self.ui.le_seqAuthor.setText(self.itemList[item['index']].author)

            ### description
            if self.itemList[item['index']].description:
                self.ui.le_seqDefinition.setText(self.itemList[item['index']].description)
            else:
                self.ui.le_seqDefinition.setText('')

            self.ui.le_seqLength.setText(str(oligo_length(self.itemList[item['index']].seq)))
            self.ui.le_seqType.setText(self.itemList[item['index']].type_)
            self.ui.le_seqDate.setText(self.itemList[item['index']].creation_date)

            self.insertAnnotations(item['index'])

            self.ui.tabWidget.setCurrentIndex(tabIndex['seq'])


        if item['type'] == seq_annot_item:

            f = self.itemList[item['index']]   ### load feature class

            ### info on parent sequence
            self.ui.le_seqName.setText(self.itemList[f.parent_id].name)
            if self.itemList[f.parent_id].longName:
                self.ui.le_longName.setText(self.itemList[f.parent_id].longName)

            if self.itemList[f.parent_id].author:
                self.ui.le_seqAuthor.setText(self.itemList[f.parent_id].author)

            if self.itemList[f.parent_id].description:
                self.ui.le_seqDefinition.setText(self.itemList[f.parent_id].description)
            self.ui.le_seqLength.setText(str(oligo_length(self.itemList[f.parent_id].seq)))
            self.ui.le_seqType.setText(self.itemList[f.parent_id].type_)
            self.ui.le_seqDate.setText(self.itemList[f.parent_id].creation_date)

            self.insertAnnotations(f.parent_id)

            self.ui.tabWidget.setCurrentIndex(tabIndex['seq'])


            max = self.ui.te_seq.verticalScrollBar().maximum()

            seq_len = len(self.itemList[f.parent_id].seq)

            featureBegin = f.begin
            #self.itemList[item['index']].featuresDict[str(self.ui.tv.selectedItems()[0].text(4))].begin ### annotation index
            #print featureBegin,type(featureBegin)
            pos = int(max / (seq_len - 1) * featureBegin - max / (seq_len - 1))

            self.ui.te_seq.verticalScrollBar().setValue(pos)

            #if type==MSA_item:


    def saveProject(self):
        """Save project in .annhyb or .pyannhyb file"""
        if self.itemList:
            if not self.projectFileName:

                filter = QString()

                while True:
                    self.projectFileName = QFileDialog.getSaveFileName(self, 'Save the current project', '', 'pyAnnHyb project (*.pyannhyb);;AnnHyb project (*.annhyb);;All files (*)', filter)

                    print 'filter', str(filter)
                    print 'project file name:', self.projectFileName
                    if not self.projectFileName:
                        return

                    #check extension
                    """
                    if not os.path.splitext(str(self.projectFileName))[1]:
                        QMessageBox.warning(self, 'pyAnnHyb', 'Please input an extension:\n.annhyb for compatibility with AnnHyb program\n.pyannhyb for no compatibility')
                    else:
                        break
                    """
                    break

            r = saveProjecttoXML(str(self.projectFileName), self.projectDescription, self.itemList)
            if not r:
                self.projectChanged = False
                self.setWindowTitle('pyAnnHyb %s (%s) - %s' % (__version__, version_date, self.projectFileName))
            else:
                if r == 'Error: Not yet implemented!':
                    QMessageBox.warning(self, "pyAnnHyb", r.split(':')[1])
                else:
                    QMessageBox.warning(self, "pyAnnHyb - Error", 'Project was not saved')

        else:
            QMessageBox.warning(self, "pyAnnHyb - Error", "This project does not contain any sequence!\nIt is not possible to save an empty project")

    def saveProjectAs(self):
        """Save project in .annhyb file or .pyannhyb file (ask user for file name)"""
        if self.itemList:
            fd = QFileDialog(self)
            filter = QString()
            while True:
                self.projectFileName = fd.getSaveFileName(self, 'Save the current project', '', 'pyAnnHyb project (*.pyannhyb);;AnnHyb project (*.annhyb);;All files (*)', filter)
                #print 'filter',str(filter)
                if not self.projectFileName:
                    return

                #check extension
                if os.path.splitext(str(self.projectFileName))[1]:
                    break
                elif str(filter):
                    self.projectFileName += str(filter)
                    break
                else:
                    QMessageBox.warning(self, 'pyAnnHyb', 'Please input an extension:\n.annhyb for compatibility with AnnHyb program\n.pyannhyb for no compatibility')
            r = saveProjecttoXML(str(self.projectFileName), self.projectDescription, self.itemList)
            if not r:
                self.projectChanged = False
                #print 'new project file name: ',self.projectFileName
                self.setWindowTitle('pyAnnHyb %s (%s) - %s' % (__version__, version_date, self.projectFileName))
            else:
                if r == 'Error: Not yet implemented!':
                    QMessageBox.warning(self, "pyAnnHyb", r.split(':')[1])
                else:
                    QMessageBox.warning(self, "pyAnnHyb - Error", "AnnHyb project file was not saved!")
        else:
            QMessageBox.warning(self, "pyAnnHyb - Error", "This project does not contain any sequence!\nIt is not possible to save an empty project")


    def openProject(self):
        """load pyannhyb or annhyb project file in dictionary"""

        response = False
        SAVE = 'Save'
        DISCARD = 'Discard'
        CANCEL = 'Cancel'

        if self.projectChanged:
            message = QMessageBox(self)
            message.setText('What to do about project unsaved changes?')
            message.setWindowTitle('pyAnnHyb')
            message.setIcon(QMessageBox.Question)
            message.addButton(SAVE, QMessageBox.AcceptRole)
            message.addButton(DISCARD, QMessageBox.DestructiveRole)
            message.addButton(CANCEL, QMessageBox.RejectRole)
            message.setDetailedText('Unsaved changes in project: ' + str(self.projectFileName))
            message.exec_()
            response = message.clickedButton().text()
            if response == SAVE:
                self.saveProject()

            # discard changes
            elif response == DISCARD:
                pass
                #self.ui.button_save.setEnabled(False)

        # if we didn't cancelled show the file dialog
        if response != CANCEL:
            fd = QFileDialog(self)
            newfile = fd.getOpenFileName(self, 'Open a project', '', 'pyAnnHyb project (*.pyannhyb);;AnnHyb project (*.annhyb);;All files (*)')
            self.tvLoad(newfile)

    def closeProject(self):
        if self.projectChanged:
            response = False
            SAVE = 'Save'
            DISCARD = 'Discard'
            CANCEL = 'Cancel'
            message = QMessageBox(self)
            message.setText('What to do about project unsaved changes?')
            message.setWindowTitle('pyAnnHyb')
            message.setIcon(QMessageBox.Question)
            message.addButton(SAVE, QMessageBox.AcceptRole)
            message.addButton(DISCARD, QMessageBox.DestructiveRole)
            message.addButton(CANCEL, QMessageBox.RejectRole)
            message.setDetailedText('Unsaved changes in project: ' + str(self.projectFileName))
            message.exec_()
            response = message.clickedButton().text()
            if response == SAVE:
                self.saveProject()

            if response == CANCEL:
                print 'close cancelled'
                return

        projectFileName = ''
        projectDescription = ''
        projectChanged = False
        self.setWindowTitle('pyAnnHyb %s (%s)' % (__version__, version_date))

        #initialize dictionaries for oligo, sequence, ma list
        self.itemList = {}

        tvItems = {}
        self.tvInit()
        self.ui.ta_oligoList.setRowCount(0)
        self.ui.ta_seqList.setRowCount(0)

        self.ui.le_oligo_seq.setText('')
        self.ui.le_salt_cc.setText('')
        self.ui.le_oligo_cc.setText('')
        self.ui.le_OligoName.setText('')
        self.ui.le_oligo_len.setText(str(0))
        #IUPAC oligo seq
        self.ui.le_IUPAC_seq.setText('')
        #GC percent
        self.ui.le_gc.setText('')
        ### MW
        self.ui.leMW.setText('')
        ### Checksum
        self.ui.leChecksum.setText('')
        ### MEC
        self.ui.leMEC.setText('')
        #rev-comp oligo seq
        self.ui.le_revcomp.setText('')
        self.ui.le_melting_temp.setText('')


        #self.oligoParamUpdate(item['index'])


        self.ui.le_seqName.setText('')
        self.ui.le_longName.setText('')
        self.ui.le_seqAuthor.setText('')
        self.ui.le_seqDefinition.setText('')
        self.ui.le_seqLength.setText('')
        self.ui.le_seqType.setText('')
        self.ui.le_seqDate.setText('')
        self.ui.te_seq.setHtml(QString(''))

        #self.ui.ta_oligoList.clear()


    def tvLoad(self, fileName):
        """check if fload items dictionary in TreeWidget TV"""
        if os.path.isfile(fileName):
            dummyDescription, dummyItemList = loadProject(fileName)
            if dummyDescription != 'bad file':
                self.projectDescription, self.itemList = dummyDescription, dummyItemList

                self.ui.le_oligo_seq.clear()
                self.ui.le_salt_cc.setText(str(defaultOligoSaltCc / 1e-3))
                self.ui.le_oligo_cc.setText(str(defaultOligoCc / 1e-9))
                self.ui.le_OligoName.clear()

                ### insert items in Tree View
                self.tvInit()

                #for index,item in self.itemList.iteritems():
                for index in self.itemList:

                    item = self.itemList[index]

                    if item.type in [oligo_item, sequence_item, MSA_item]:

                        newItem = QTreeWidgetItem(catList[catType[-item.type]]['item'], [item.name, str(index), str(item.type)])

                        tvItems[index] = newItem

                for index in self.itemList:

                    item = self.itemList[index]

                    if item.type in [seq_annot_item]:

                        newItem = QTreeWidgetItem(tvItems[item.parent_id], [item.name, str(index), str(item.type)])

                        colorFeature = QColor(item.color[0], item.color[1], item.color[2], item.color[3])
                        newItem.setBackgroundColor(0, colorFeature)

                        tvItems[index] = newItem


                    """
                    if item.type == sequence_item:  ### if seq insert annotations
                        
                        ds={}
                        for featureIndex in item.featuresDict:
                            ds[item.featuresDict[featureIndex].begin]=featureIndex
                        print ds    


                        for featureIndex in item.featuresDict:
                            feature=item.featuresDict[featureIndex]
                            
                            newFeature=QTreeWidgetItem(newItem,[feature.name,'None',str(seqAnnotation),str(index),str(featureIndex),'%09d' % feature.begin])
                            #print newFeature.text(0)
                            tvItems[featureIndex]=newFeature
                            ### set background color for annotation
                            colorFeature=QColor(feature.color[0],feature.color[1],feature.color[2],feature.color[3])
                            newFeature.setBackgroundColor(0,colorFeature)
                        
                        tvItems[index].sortChildren(5,Qt.AscendingOrder)
                        #print tvItems[index].childCount()
                    """

                ### expand items
                for cat in catList:
                     self.ui.tv.expandItem(catList[cat]['item'])

                ### select oligo list by default
                self.ui.tv.setItemSelected(catList['oligoList']['item'], True)

                self.projectFileName = fileName
                self.projectChanged = False
                self.setWindowTitle('pyAnnHyb %s (%s) - %s' % (__version__, version_date, self.projectFileName))

            else: ### bad file
                QMessageBox.warning(self, 'pyAnnHyb', 'The file <b>%s</b> is not a pyannnyb/AnnHyb project file!' % fileName)
        else: ### file do not exists        
            QMessageBox.warning(self, 'pyAnnHyb', 'The file <b>%s</b> do not exists!' % fileName)

    def removeItem(self):
        """remove the selected item from project"""
        """TODO: add remove all items function (when items List selected)"""
        item = self.checkItem()
        if item['type'] in [oligo_item, sequence_item]:
            self.ui.tv.removeItemWidget(tvItems[item['index']], 0)
            del tvItems[item['index']]
            self.itemList.pop(item['index'])
            self.projectChanged = True

        elif item['type'] in [oligoList]:
            if self.ynMessageBox('Do you want to remove all oligonucleotides?', '') == 'Yes':
                for index in self.itemList.keys():
                    item = self.itemList[index]
                    print index, item
                    if item.type == oligo_item:
                        self.ui.tv.removeItemWidget(tvItems[index], 0)
                        del tvItems[index]
                        self.itemList.pop(index)
                self.fillOligoList()
                self.projectChanged = True

        elif item['type'] in [seqList]:
            if self.ynMessageBox('Do you want to remove all sequences?', '') == 'Yes':
                for index in self.itemList.keys():
                    item = self.itemList[index]
                    print index, item
                    if item.type == sequence_item:
                        self.ui.tv.removeItemWidget(tvItems[index], 0)
                        del tvItems[index]
                        self.itemList.pop(index)
                self.fillSeqList()
                self.projectChanged = True
        else:
            QMessageBox.warning(self, "pyAnnHyb", 'No selected item')

    def exportOligoFASTAFormat(self):
        """Export all oligonucleotides in FASTA format"""
        countOligo = 0
        for i in self.itemList.keys():
            if self.itemList[i].type == oligo_item:
                countOligo += 1
        if countOligo:
            fd = QFileDialog(self)
            newfile = fd.getSaveFileName(self, 'Export oligo in FASTA format', '', 'All files (*)')
            if newfile:
                f = open(newfile, 'w')
                for i in self.itemList.keys():
                    if self.itemList[i].type == oligo_item:
                        f.write('>' + self.itemList[i].name + self.NL + convert2IUPAC(self.itemList[i].seq).replace(' ', '') + self.NL)
                f.close()
        else:
            QMessageBox.warning(self, "pyAnnHyb", 'No oligonucleotides to export')

    def exportOligoData(self):
        """Export oligo data"""
        fd = QFileDialog(self)
        newfile = fd.getSaveFileName(self, 'Export oligo data', '', 'All files (*)')
        if newfile:
            f = open(newfile, 'w')
            for i in self.itemList.keys():
                if self.itemList[i].type == oligo_item:
                    f.write('Name: %s' % (self.itemList[i].name) + self.NL)
                    f.write('Sequence: %s' % (convert2IUPAC(self.itemList[i].seq).replace(' ', '')) + self.NL * 2)
                    f.write('Reverted/complemented: %s' % (rev_comp_seq(convert2IUPAC(self.itemList[i].seq).replace(' ', ''))) + self.NL)
                    f.write('Length: %d b' % (oligo_length(convert2IUPAC(self.itemList[i].seq))) + self.NL)
                    f.write('Molecular weight: %d' % (MW(self.itemList[i].seq)) + self.NL)
                    f.write('GC percent: %.1f' % (GCp(self.itemList[i].seq) * 100) + self.NL)
                    f.write('GCG checksum: %s' % (GCG_checksum(self.itemList[i].seq)) + self.NL)
                    f.write('SEGUID checksum: %s' % (seguid(self.itemList[i].seq)) + self.NL)
                    print type(self.preferences['oligoDefaultSaltConc'])
                    f.write('Tm (%.1f mM salt; %.1f nM oligonucleotide): %.1f C' % (float(self.preferences['oligoDefaultSaltConc']) * 1e3, float(self.preferences['oligoDefaultConc']) * 1e9, nn_melting(self.itemList[i].seq, float(self.preferences['oligoDefaultSaltConc']), float(self.preferences['oligoDefaultConc']))[0]) + self.NL)
                    f.write(self.NL * 3)
            f.close()

    def newOligo(self):
        """Create a new oligo"""

        try:
            item_id = len(self.itemList) + 1
        except:
            item_id = 0

        #init new oligo
        o = Oligo()

        ### check for default name
        i = 0
        while True:
            i += 1
            o.name = 'New oligo ' + str(i)
            flag = False
            for j in self.itemList.keys():
                if self.itemList[j].type == oligo_item and self.itemList[j].name == o.name:
                    flag = True
            if not flag:
                break


        o.type = oligo_item
        o.seq = ''
        o.salt_cc = float(self.preferences['oligoDefaultSaltConc'])
        o.oligo_cc = float(self.preferences['oligoDefaultConc'])
        o.notes = ''

        self.itemList[item_id] = o

        if self.ui.tv.selectedItems():
            self.ui.tv.selectedItems()[0].setSelected(False)   #deselect previous selection

        oligo = QTreeWidgetItem(catList['oligoList']['item'], [self.itemList[item_id].name, str(item_id), str(oligo_item)]) #set oligo name and index
        tvItems[item_id] = oligo

        oligo.setSelected(True)  #select new oligo
        self.ui.tv.expandItem(catList['oligoList']['item'])
        self.projectChanged = True

    def newSequence(self):
        """input a new sequence"""

        frm_newSeq = NewSeq()
        frm_newSeq.setWindowTitle(QString('Input a new sequence'))
        frm_newSeq.ui.teSeq.setText(QString(''))
        frm_newSeq.exec_()

        newseq = clear_seq(frm_newSeq.ui.teSeq.toPlainText())
        print newseq

        s = Sequence()

        ### check for default name
        i = 0
        while True:
            i += 1
            s.name = 'New sequence ' + str(i)
            flag = False
            for j in self.itemList.keys():
                if self.itemList[j].type == sequence_item and self.itemList[j].name == s.name:
                    flag = True
            if not flag:
                break

        s.seq = str(newseq)

        id = len(self.itemList) + 1
        self.itemList[id] = s

        seqitem = QTreeWidgetItem(catList['seqList']['item'], [self.itemList[id].name, str(id), str(sequence_item)]) #set seq name and index

        tvItems[id] = seqitem

        ### deselect previous selection
        if self.ui.tv.selectedItems():
            self.ui.tv.selectedItems()[0].setSelected(False)

        seqitem.setSelected(True)

        self.ui.le_seqName.setText(self.itemList[id].name)
        self.insertAnnotations(id)

        self.projectChanged = True


    def oligoSeq_changed(self):
        print 'oligo seq changed'
        #update TreeWidget
        i = int(self.ui.tv.selectedItems()[0].text(1))
        self.itemList[i].seq = self.ui.le_oligo_seq.text()

        self.oligoParamUpdate(i)
        self.projectChanged = True

    def oligoSaltCc_changed(self):
        print 'Oligo salt cc changed'
        i = int(self.ui.tv.selectedItems()[0].text(1))
        self.itemList[i].salt_cc = float(self.ui.le_salt_cc.text()) * 1e-3
        self.oligoParamUpdate(i)
        self.projectChanged = True

    def oligoCc_changed(self):
        print 'Oligo cc changed'
        i = int(self.ui.tv.selectedItems()[0].text(1))
        self.itemList[i].oligo_cc = float(self.ui.le_oligo_cc.text()) * 1e-9
        self.oligoParamUpdate(i)
        self.projectChanged = True

    def revcomp_changed(self):
        print 'rev comp seq changed'
        self.ui.le_oligo_seq.setText(rev_comp_seq(str(self.ui.le_revcomp.text())))
        self.oligoSeq_changed()

    def quit_application(self):
        self.close()

    def about(self):
#        frm_re_selection=RESelection()
#        frm_re_selection.exec_()

##        help_=frmHelp()
##        help_.exec_()
##        print help_.result()
##        print help_.ui.lineEdit.text()


##        message=QMessageBox(self)
##        message.setWindowTitle('About PyAnnHyb')
##        message.setText('The goal of the PyAnnHyb project is to rewrite the complete AnnHyb application with the Python programming language.\nThe current version '+version+' is at an early stage of development.\nSee http://bioinformatics.org/annhyb/pyannhyb for more details.\n\nCopyright Olivier Friard 2008.')
##        message.setIcon(QMessageBox.Information)
##        message.exec_()

#        message.addButton(SAVE, QtGui.QMessageBox.AcceptRole)
#		message.addButton(DISCARD, QtGui.QMessageBox.DestructiveRole)
#		message.addButton(CANCEL, QtGui.QMessageBox.RejectRole)
#        message.setDetailedText('Unsaved changes in file: ')
##        message.exec_()
#        response = message.clickedButton().text()



        QMessageBox.about(self, "About pyAnnHyb",
        """<b>PyAnnHyb</b> v. %s
        <p>Copyright &copy; 2008-2011 Olivier Friard.
        All rights reserved.
        <p>The goal of the PyAnnHyb project is to rewrite the complete AnnHyb application with the Python programming language.<br>
        This version is at an EARLY stage of development. Use it at your own risks!<br>
        See <a href="http://www.bioinformatics.org/pyannhyb">www.bioinformatics.org/pyannhyb</a> for more details.
        <p>Python %s - Qt %s - PyQt %s on %s<br>Biopython v. %s""" % (__version__, platform.python_version(), QT_VERSION_STR, PYQT_VERSION_STR, platform.system(), biopythonVersion))
#
if __name__ == "__main__":
    app = QApplication(sys.argv)
    annhyb = AnnHyb()

    #check if Biopython is installed
    try:
        from Bio import GenBank
    except ImportError:
        QMessageBox.warning(annhyb, "pyAnnHyb", '<b>Biopython</b> module is required (version >=1.50)<br>See <a href="http://biopython.org">Biopython project</a>')
        sys.exit(1)

    #check biopython version
    try:
        import Bio
        biopythonVersion = Bio.__version__
        if float(biopythonVersion.replace('b', '')) < 1.49:
            QMessageBox.warning(annhyb, "pyAnnHyb", '<b>Biopython</b> version >=1.49 is required<br>See <a href="http://biopython.org">Biopython project</a>')
            sys.exit(1)
    except AttributeError:
        #check biopython version using Martel module
        import Martel
        biopythonVersion = Martel.__version__
        if float(biopythonVersion.replace('b', '')) < 1.49:
            QMessageBox.warning(annhyb, "pyAnnHyb", '<b>Biopython</b> version >=1.49 is required<br>See <a href="http://biopython.org">Biopython project</a>')
            sys.exit(1)

    from Bio.Seq import Seq
    from Bio.Alphabet.IUPAC import IUPACAmbiguousDNA
    from Bio.Restriction import *
    from Bio import SeqIO

    #check if argument
    if len(sys.argv) > 1:
        annhyb.tvLoad(sys.argv[1])

    annhyb.show()
    sys.exit(app.exec_())
