#!/usr/bin/env python
"""
save_project.py - PyAnnHyb
     Copyright (C) 2008-2011 Olivier Friard

  This file is part of PyAnnHyb.

PyAnnHyb is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

PyAnnHyb is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with PyAnnHyb; see the file COPYING.TXT.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
"""

import os
from seq_classes import *

def saveProjecttoXML(filename, projectDescription, itemList):

    ### compatibility with AnnHyb project format            
    if os.path.splitext(filename)[1] == '.annhyb':

        import xml.etree.ElementTree as ET
        root = ET.Element("annhyb")

        project = ET.SubElement(root, "project")
        description = ET.SubElement(project, "description")
        description.text = projectDescription

        oligos = ET.SubElement(root, "oligonucleotides")
        sequences = ET.SubElement(root, "sequences")
        ma_s = ET.SubElement(root, "multi-alignments")

        #for k,item in itemList.iteritems():
        for k in itemList:

            item = itemList[k]

            if item.type == oligo_item:
                oligo = ET.SubElement(oligos, "oligo")

                oligo_name = ET.SubElement(oligo, "oligo_name")
                oligo_name.text = str(item.name)

                oligo_seq = ET.SubElement(oligo, "oligo_seq")
                oligo_seq.text = str(item.seq)

                oligo_notes = ET.SubElement(oligo, "oligo_notes")
                oligo_notes.text = str(item.notes)

                oligo_salt_cc = ET.SubElement(oligo, "oligo_salt_concentration")
                oligo_salt_cc.text = str(item.salt_cc)

                oligo_cc = ET.SubElement(oligo, "oligo_concentration")
                oligo_cc.text = str(item.oligo_cc)

            if item.type == sequence_item:
                seq = ET.SubElement(sequences, "sequence")

                if item.name:
                    xml_el = ET.SubElement(seq, "sequence_name")
                    xml_el.text = str(item.name)

                if item.longName:
                    xml_el = ET.SubElement(seq, "sequence_longname")
                    xml_el.text = str(item.longName)

                if item.description:
                    xml_el = ET.SubElement(seq, "sequence_description")
                    xml_el.text = str(item.description)

                if item.type_:
                    xml_el = ET.SubElement(seq, "sequence_type")
                    xml_el.text = str(item.type_)

                if item.author:
                    xml_el = ET.SubElement(seq, "sequence_author")
                    xml_el.text = str(item.author)

                if item.creation_date:
                    xml_el = ET.SubElement(seq, "sequence_creation_date")
                    xml_el.text = str(item.creation_date)

                if item.header:
                    xml_el = ET.SubElement(seq, "sequence_header")
                    xml_el.text = str(item.header)

                str_features = ''
                features = ET.SubElement(seq, "sequence_features")
                for j in itemList:
                    if itemList[j].type == seq_annot_item:
                        if itemList[j].parent_id == k:

                            f = itemList[j]

                            Tcolor = f.color[0] | (f.color[1] << 8) | (f.color[2] << 16)
                            if not Tcolor in (0, 128, 32768, 32896, 8388608, 8388736, 8421376, 8421504, 12632256, 255, 65280, 65535, 16711680, 16711935, 16776960, 16777215):
                                Tcolor = 65535  #set to yellow   
                            str_features += 'feature  ' + str(f.begin) + '  ' + str(f.end) + '  ' + str(Tcolor) + ' square solid ' + str(f.name) + '///' + str(f.comment) + '\n'
                            features.text = str_features
                            """
                            feature=ET.SubElement(features,"feature")

                            xml_el=ET.SubElement(feature,"name")
                            xml_el.text=str(f.name)
    
                            xml_el=ET.SubElement(feature,"begin")
                            xml_el.text=str(f.begin)
    
                            xml_el=ET.SubElement(feature,"end")
                            xml_el.text=str(f.end)
    
                            xml_el=ET.SubElement(feature,"comment")
                            xml_el.text=str(f.comment)
    
                            xml_el=ET.SubElement(feature,"color")
                            xml_el.text=str(f.color)
                            """

                """    
                if item.featuresDict:
                    features=ET.SubElement(seq,"sequence_features")
                    str_features=''
                    for featureIndex in item.featuresDict:
                        f=item.featuresDict[featureIndex]

#                    for f in item.features:
                        #convert RGB color to Delphi color
                        Tcolor=f.color[0] | (f.color[1] <<8) | (f.color[2] <<16)
                        if not Tcolor in (0,128,32768,32896,8388608,8388736,8421376,8421504,12632256,255,65280,65535,16711680,16711935,16776960,16777215):
                           Tcolor = 65535  #set to yellow   
                        str_features+='feature  '+str(f.begin)+'  '+str(f.end)+'  '+str(Tcolor)+' square solid '+str(f.name)+'///'+str(f.comment)+'\n'
                    features.text=str_features    
                """


                if item.seq:
                    xml_el = ET.SubElement(seq, "sequence_seq")
                    xml_el.text = str(item.seq)

            if item.type == MSA_item:
                ma = ET.SubElement(ma_s, "multi-alignment")

                xml_el = ET.SubElement(ma, "ma_name")
                xml_el.text = str(item.name)

                xml_el = ET.SubElement(ma, "ma_seq_name")
                xml_el.text = str(item.sequences_names)

                xml_el = ET.SubElement(ma, "ma_seq_seq")
                xml_el.text = str(item.sequences)



    ### format pyannhyb
    else:   # os.path.splitext(filename)[1] == '.pyannhyb':
        import xml.etree.ElementTree as ET
        root = ET.Element("pyannhyb")

        project = ET.SubElement(root, "project")
        description = ET.SubElement(project, "description")
        description.text = projectDescription

        oligos = ET.SubElement(root, "oligonucleotides")
        sequences = ET.SubElement(root, "sequences")
        ma_s = ET.SubElement(root, "multi-alignments")

        for k in itemList:

            item = itemList[k]

            if item.type == oligo_item:
                oligo = ET.SubElement(oligos, "oligo")

                oligo_name = ET.SubElement(oligo, "oligo_name")
                oligo_name.text = str(item.name)

                oligo_seq = ET.SubElement(oligo, "oligo_seq")
                oligo_seq.text = str(item.seq)

                oligo_notes = ET.SubElement(oligo, "oligo_notes")
                oligo_notes.text = str(item.notes)

                oligo_salt_cc = ET.SubElement(oligo, "oligo_salt_concentration")
                oligo_salt_cc.text = str(item.salt_cc)

                oligo_cc = ET.SubElement(oligo, "oligo_concentration")
                oligo_cc.text = str(item.oligo_cc)

            if item.type == sequence_item:
                seq = ET.SubElement(sequences, "sequence")

                if item.name:
                    xml_el = ET.SubElement(seq, "sequence_name")
                    xml_el.text = str(item.name)

                if item.longName:
                    xml_el = ET.SubElement(seq, "sequence_longname")
                    xml_el.text = str(item.longName)

                if item.description:
                    xml_el = ET.SubElement(seq, "sequence_description")
                    xml_el.text = str(item.description)

                if item.type_:
                    xml_el = ET.SubElement(seq, "sequence_type")
                    xml_el.text = str(item.type_)

                if item.author:
                    xml_el = ET.SubElement(seq, "sequence_author")
                    xml_el.text = str(item.author)

                if item.creation_date:
                    xml_el = ET.SubElement(seq, "sequence_creation_date")
                    xml_el.text = str(item.creation_date)

                if item.header:
                    xml_el = ET.SubElement(seq, "sequence_header")
                    xml_el.text = str(item.header)

                features = ET.SubElement(seq, "features")
                for j in itemList:
                    if itemList[j].type == seq_annot_item:
                        if itemList[j].parent_id == k:

                            f = itemList[j]
                            feature = ET.SubElement(features, "feature")

                            xml_el = ET.SubElement(feature, "name")
                            xml_el.text = str(f.name)

                            xml_el = ET.SubElement(feature, "begin")
                            xml_el.text = str(f.begin)

                            xml_el = ET.SubElement(feature, "end")
                            xml_el.text = str(f.end)

                            xml_el = ET.SubElement(feature, "comment")
                            xml_el.text = str(f.comment)

                            xml_el = ET.SubElement(feature, "color")
                            xml_el.text = str(f.color)



                """
                if item.featuresDict:
                    features=ET.SubElement(seq,"features")
                    
                    for featureIndex in item.featuresDict:
                        f=item.featuresDict[featureIndex]
                        
                        feature=ET.SubElement(features,"feature")
                        xml_el=ET.SubElement(feature,"name")
                        xml_el.text=str(f.name)
    
                        xml_el=ET.SubElement(feature,"begin")
                        xml_el.text=str(f.begin)
    
                        xml_el=ET.SubElement(feature,"end")
                        xml_el.text=str(f.end)
    
                        xml_el=ET.SubElement(feature,"comment")
                        xml_el.text=str(f.comment)
    
                        xml_el=ET.SubElement(feature,"color")
                        xml_el.text=str(f.color)
                """

                if item.seq:
                    xml_el = ET.SubElement(seq, "sequence_seq")
                    xml_el.text = str(item.seq)

            if item.type == MSA_item:
                ma = ET.SubElement(ma_s, "multi-alignment")

                xml_el = ET.SubElement(ma, "ma_name")
                xml_el.text = str(item.name)

                xml_el = ET.SubElement(ma, "ma_seq_name")
                xml_el.text = str(item.sequences_names)

                xml_el = ET.SubElement(ma, "ma_seq_seq")
                xml_el.text = str(item.sequences)



    #if os.path.splitext(filename)[1] in [',pyannhyb', '.annhyb']:
    tree = ET.ElementTree(root)
    from xml.etree.ElementTree import XML, fromstring, tostring

    try:
        f = open(filename, 'w')
        s2 = tostring(root, "ISO-8859-15")
        s2 = s2.replace('><', '>\n<')
        f.write(s2)
        return 0
    except:
        return 1
